<?php
/**
 * Brainy Internal Plugin Templateparser
 *
 * This is the template parser.
 * It is generated from the smarty_internal_templateparser.y file
 * @package Brainy
 * @subpackage Compiler
 * @author Uwe Tews
 * @author Matt Basta
 */

class BrainySafeLookupWrapper
{
    private $unsafeVersion;
    private $safeVersion;

    /**
     * @param string $unsafe
     * @param string $safe
     */
    public function __construct($unsafe, $safe) {
        $this->unsafeVersion = $unsafe;
        $this->safeVersion = $safe;
    }

    /**
     * @return string
     */
    public function getUnsafe() {
        return $this->unsafeVersion;
    }

    /**
     * The __toString() method should always return the safe version.
     * @return string
     */
    public function __toString() {
        return $this->safeVersion;
    }
}

class BrainyStaticWrapper
{
    private $code;

    /**
     * @param string $code
     */
    public function __construct($code) {
        $this->code = $code;
    }

    /**
     * The __toString() method un-statics the contents
     * @return string
     */
    public function __toString() {
        return $this->code;
    }

    /**
     * Combine two values which may be static wrappers. The output is only a
     * static wrapper if both the left and the right are static wrappers.
     * @param string|BrainyStaticWrapper $left
     * @param string|BrainyStaticWrapper $right
     * @return string|BrainyStaticWrapper
     */
    public static function concat($left, $right) {
        if ($left instanceof BrainyStaticWrapper && $right instanceof BrainyStaticWrapper) {
            return new BrainyStaticWrapper($left . $right);
        }
        return $left . $right;
    }

    /**
     * Combine two values which may be static wrappers. The output is a static
     * wrapper if either the left or the right are static wrappers. This is
     * only to be used in cases where the left or the right is known not to
     * affect whether the result is a static wrapper.
     * @param string|BrainyStaticWrapper $left
     * @param string|BrainyStaticWrapper $right
     * @return string|BrainyStaticWrapper
     */
    public static function static_concat($left, $right) {
        if ($left instanceof BrainyStaticWrapper || $right instanceof BrainyStaticWrapper) {
            return new BrainyStaticWrapper($left . $right);
        }
        return $left . $right;
    }

    /**
     * If all of the values in $conditions are static wrappers, the result is a
     * static wrapper of $code. Otherwise, $code is returned.
     * @param string $code
     * @param array $conditions
     * @return string|BrainyStaticWrapper
     */
    public static function static_if_all($code, $conditions) {
        foreach ($conditions as $cond) {
            if (!($cond instanceof BrainyStaticWrapper)) {
                return $code;
            }
        }
        return new BrainyStaticWrapper($code);
    }
}

class TP_yyStackEntry
{
    public $stateno;       /* The state-number */
    public $major;         /* The major token value.  This is the code
                     ** number for the token at this stack level */
    public $minor; /* The user-supplied minor token value.  This
                     ** is the value of the token  */
};


#line 17 "smarty_internal_templateparser.y"
class Smarty_Internal_Templateparser#line 112 "smarty_internal_templateparser.php"
{
#line 19 "smarty_internal_templateparser.y"

    const Err1 = "Security error: Call to private object member not allowed";
    const Err2 = "Security error: Call to dynamic object member not allowed";
    // states whether the parse was successful or not
    public $successful = true;
    public $retvalue = 0;
    public static $prefix_number = 0;
    private $lex;
    private $internalError = false;
    private $strip = 0;

    private $safe_lookups = 0;
    private $strict_mode = false;

    function __construct($lex, $compiler) {
        $this->lex = $lex;
        $this->compiler = $compiler;
        $this->smarty = $this->compiler->smarty;
        $this->template = $this->compiler->template;
        $this->compiler->has_variable_string = false;
        $this->compiler->prefix_code = array();
        $this->security = isset($this->smarty->security_policy);
        $this->block_nesting_level = 0;
        $this->current_buffer = $this->root_buffer = new _smarty_template_buffer($this);

        $this->safe_lookups = $this->smarty->safe_lookups;
    }

    /**
     * Strips whitespace from a string
     * @param string $string
     * @return string
     */
    protected static function stripString($string) {
        // Replaces whitespace followed by a `<` with null.
        // `     \n     <foo>` -> `<foo>`
        $string = preg_replace('/\s+(?=<)/ims', '', $string);
        // Replaces `>` followed by whitespace with `>`
        // `<div>\n  foo` -> `<div>foo`
        $string = preg_replace('/>\s+(?=\S)/ims', '>', $string);
        // Replaces whitespace followed by anything else with a space.
        // `            data-hello="` -> ` data-hello="`
        $string = preg_replace('/\s+(?=\w)/ims', ' ', $string);

        // Is there work to be done at the end of the string?
        if ($string !== rtrim($string)) {
            $string = rtrim($string);
            // If the last non-whitespace character is not a `>`, add a space.
            if (substr($string, -1) !== '>') {
                $string .= ' ';
            }
        }
        return $string;
    }

    /**
     * @return bool
     */
    public function isStrictMode() {
        return $this->strict_mode;
    }

    /**
     * @param string $variable The name of the variable to look up
     * @param string|void $value The member of the SmartyVariable to access
     * @return string|BrainySafeLookupWrapper
     */
    public function compileVariable($variable, $value = 'value')
    {
        $unsafe = '$_smarty_tpl->tpl_vars[' . $variable . ']->' . $value;
        if ($this->safe_lookups === 0) { // Unsafe lookups
            return $unsafe;
        }
        $safe = 'smarty_safe_var_lookup($_smarty_tpl->tpl_vars, '. $variable .', ' . $this->safe_lookups . ')->' . $value;
        return new BrainySafeLookupWrapper($unsafe, $safe);
    }

    /**
     * @param string $base
     * @param string $variable
     * @return string|BrainySafeLookupWrapper
     */
    public function compileSafeLookupWithBase($base, $variable)
    {
        $unsafe = $base . '[' . $variable . ']';
        if ($this->safe_lookups === 0) { // Unsafe lookups
            return $unsafe;
        }
        $safe = 'smarty_safe_array_lookup(' . $base . ', '. $variable .', ' . $this->safe_lookups . ')';
        return new BrainySafeLookupWrapper($unsafe, $safe);
    }
#line 207 "smarty_internal_templateparser.php"

    const TP_VERT                           =  1;
    const TP_COLON                          =  2;
    const TP_SETSTRICT                      =  3;
    const TP_RDEL                           =  4;
    const TP_COMMENT                        =  5;
    const TP_TEXT                           =  6;
    const TP_STRIPON                        =  7;
    const TP_STRIPOFF                       =  8;
    const TP_BLOCKSOURCE                    =  9;
    const TP_LITERALSTART                   = 10;
    const TP_LITERALEND                     = 11;
    const TP_LITERAL                        = 12;
    const TP_LDEL                           = 13;
    const TP_EQUAL                          = 14;
    const TP_DOLLAR                         = 15;
    const TP_ID                             = 16;
    const TP_LDELIF                         = 17;
    const TP_LDELFOR                        = 18;
    const TP_SEMICOLON                      = 19;
    const TP_INCDEC                         = 20;
    const TP_TO                             = 21;
    const TP_STEP                           = 22;
    const TP_LDELFOREACH                    = 23;
    const TP_SPACE                          = 24;
    const TP_AS                             = 25;
    const TP_APTR                           = 26;
    const TP_LDELSETFILTER                  = 27;
    const TP_SMARTYBLOCKCHILDPARENT         = 28;
    const TP_LDELSLASH                      = 29;
    const TP_ATTR                           = 30;
    const TP_INTEGER                        = 31;
    const TP_COMMA                          = 32;
    const TP_OPENP                          = 33;
    const TP_CLOSEP                         = 34;
    const TP_MATH                           = 35;
    const TP_UNIMATH                        = 36;
    const TP_ANDSYM                         = 37;
    const TP_ISIN                           = 38;
    const TP_ISDIVBY                        = 39;
    const TP_ISNOTDIVBY                     = 40;
    const TP_ISEVEN                         = 41;
    const TP_ISNOTEVEN                      = 42;
    const TP_ISEVENBY                       = 43;
    const TP_ISNOTEVENBY                    = 44;
    const TP_ISODD                          = 45;
    const TP_ISNOTODD                       = 46;
    const TP_ISODDBY                        = 47;
    const TP_ISNOTODDBY                     = 48;
    const TP_INSTANCEOF                     = 49;
    const TP_QMARK                          = 50;
    const TP_NOT                            = 51;
    const TP_TYPECAST                       = 52;
    const TP_DOT                            = 53;
    const TP_SINGLEQUOTESTRING              = 54;
    const TP_AT                             = 55;
    const TP_HATCH                          = 56;
    const TP_OPENB                          = 57;
    const TP_CLOSEB                         = 58;
    const TP_PTR                            = 59;
    const TP_EQUALS                         = 60;
    const TP_NOTEQUALS                      = 61;
    const TP_GREATERTHAN                    = 62;
    const TP_LESSTHAN                       = 63;
    const TP_GREATEREQUAL                   = 64;
    const TP_LESSEQUAL                      = 65;
    const TP_IDENTITY                       = 66;
    const TP_NONEIDENTITY                   = 67;
    const TP_MOD                            = 68;
    const TP_LAND                           = 69;
    const TP_LOR                            = 70;
    const TP_LXOR                           = 71;
    const TP_QUOTE                          = 72;
    const TP_DOLLARID                       = 73;
    const YY_NO_ACTION = 501;
    const YY_ACCEPT_ACTION = 500;
    const YY_ERROR_ACTION = 499;

    const YY_SZ_ACTTAB = 2196;
static public $yy_action = array(
 /*     0 */   195,  173,   40,  291,  116,  284,  271,  267,  268,  269,
 /*    10 */   274,  174,  166,  184,    8,   15,   34,    9,    6,  103,
 /*    20 */   308,  500,   90,   39,  130,  181,  263,  184,  219,   22,
 /*    30 */   201,    9,  195,  149,   51,   45,   46,   43,   28,   35,
 /*    40 */   306,  303,   41,   42,  304,  248,   37,   17,   12,   20,
 /*    50 */   184,  266,    5,  307,    9,  247,  149,   29,  293,  305,
 /*    60 */   310,  311,  317,  318,  316,  315,  312,  313,  314,  302,
 /*    70 */   301,  195,  203,   12,   24,  210,  266,  242,  299,  143,
 /*    80 */    47,   12,   24,  213,  266,   10,  174,  281,  279,    6,
 /*    90 */   103,  159,   21,  233,   39,  130,  250,  262,  111,  219,
 /*   100 */    22,  201,  195,    2,   11,   51,   45,   46,   43,   28,
 /*   110 */    35,  306,  303,   41,   42,  304,  248,   37,   17,  195,
 /*   120 */   272,  202,  270,  204,   93,  230,  202,  270,  180,  298,
 /*   130 */   305,  310,  311,  317,  318,  316,  315,  312,  313,  314,
 /*   140 */   302,  301,   39,  184,  286,  292,  195,    9,   22,  378,
 /*   150 */    47,  133,  205,   51,   45,   46,   43,   28,   35,  306,
 /*   160 */   303,   41,   42,  304,  248,   37,   17,  195,  163,  378,
 /*   170 */   215,   38,  102,  115,  188,  378,  144,  277,  305,  310,
 /*   180 */   311,  317,  318,  316,  315,  312,  313,  314,  302,  301,
 /*   190 */   263,  224,  113,  195,   47,  195,  349,   12,  378,  170,
 /*   200 */   266,   51,   45,   46,   43,   28,   35,  306,  303,   41,
 /*   210 */    42,  304,  248,   37,   17,  195,   39,  195,  378,   33,
 /*   220 */   339,  246,   22,  181,  378,    5,  305,  310,  311,  317,
 /*   230 */   318,  316,  315,  312,  313,  314,  302,  301,  199,  209,
 /*   240 */    39,  177,  195,   47,   12,  345,   22,  266,  240,   51,
 /*   250 */    45,   46,   43,   28,   35,  306,  303,   41,   42,  304,
 /*   260 */   248,   37,   17,  195,  195,   47,  295,  375,   32,  195,
 /*   270 */   171,  104,  300,  176,  305,  310,  311,  317,  318,  316,
 /*   280 */   315,  312,  313,  314,  302,  301,  204,  375,    3,   33,
 /*   290 */    47,  319,  195,  375,  181,  265,  178,   51,   45,   46,
 /*   300 */    43,   28,   35,  306,  303,   41,   42,  304,  248,   37,
 /*   310 */    17,  195,   47,  195,  234,   39,  372,   33,  241,  296,
 /*   320 */   181,   22,  305,  310,  311,  317,  318,  316,  315,  312,
 /*   330 */   313,  314,  302,  301,  214,  189,   39,   39,   18,  258,
 /*   340 */   259,  208,   22,   22,  253,   51,   45,   46,   43,   28,
 /*   350 */    35,  306,  303,   41,   42,  304,  248,   37,   17,  100,
 /*   360 */   169,   27,  280,   12,  282,   36,  266,    7,    5,  277,
 /*   370 */   305,  310,  311,  317,  318,  316,  315,  312,  313,  314,
 /*   380 */   302,  301,  195,  181,  179,  210,  101,  158,   12,  122,
 /*   390 */   159,  266,   74,  213,  249,   98,  277,  108,  175,   39,
 /*   400 */   172,  159,  251,  290,  212,   22,  250,  262,  111,  263,
 /*   410 */   181,  121,   12,  196,  212,  211,   51,   45,   46,   43,
 /*   420 */    28,   35,  306,  303,   41,   42,  304,  248,   37,   17,
 /*   430 */   195,  162,   12,  232,    5,  207,   12,  131,   26,  228,
 /*   440 */   277,  305,  310,  311,  317,  318,  316,  315,  312,  313,
 /*   450 */   314,  302,  301,   12,  180,  123,  190,   19,  195,   44,
 /*   460 */   129,  253,  255,   22,   51,   45,   46,   43,   28,   35,
 /*   470 */   306,  303,   41,   42,  304,  248,   37,   17,  195,  156,
 /*   480 */   198,  236,  237,  120,  128,  126,  117,  208,  277,  305,
 /*   490 */   310,  311,  317,  318,  316,  315,  312,  313,  314,  302,
 /*   500 */   301,  227,  180,   13,  243,  254,  257,  244,  273,   16,
 /*   510 */   200,  208,   51,   45,   46,   43,   28,   35,  306,  303,
 /*   520 */    41,   42,  304,  248,   37,   17,  195,  165,  142,  256,
 /*   530 */   206,  252,  283,  261,   11,  148,  277,  305,  310,  311,
 /*   540 */   317,  318,  316,  315,  312,  313,  314,  302,  301,  263,
 /*   550 */   180,  151,  276,  238,  231,  278,  260,  235,  229,  225,
 /*   560 */    51,   45,   46,   43,   28,   35,  306,  303,   41,   42,
 /*   570 */   304,  248,   37,   17,  195,   30,  265,  226,   14,  275,
 /*   580 */    44,  295,  110,   95,  147,  305,  310,  311,  317,  318,
 /*   590 */   316,  315,  312,  313,  314,  302,  301,  263,  263,  295,
 /*   600 */   295,  295,  295,  295,  295,  295,  295,  295,   51,   45,
 /*   610 */    46,   43,   28,   35,  306,  303,   41,   42,  304,  248,
 /*   620 */    37,   17,  195,  164,  295,  295,  295,  295,  295,  295,
 /*   630 */   295,  109,  277,  305,  310,  311,  317,  318,  316,  315,
 /*   640 */   312,  313,  314,  302,  301,  263,  199,  295,  295,  295,
 /*   650 */   295,  295,  295,  295,  295,  295,   51,   45,   46,   43,
 /*   660 */    28,   35,  306,  303,   41,   42,  304,  248,   37,   17,
 /*   670 */   195,  295,  295,  295,  295,  295,  295,  295,  295,  245,
 /*   680 */   287,  305,  310,  311,  317,  318,  316,  315,  312,  313,
 /*   690 */   314,  302,  301,  295,  167,  295,  295,  295,  295,  295,
 /*   700 */   295,  295,  295,  277,   51,   45,   46,   43,   28,   35,
 /*   710 */   306,  303,   41,   42,  304,  248,   37,   17,  295,  295,
 /*   720 */   295,  295,  295,  295,  295,  295,  295,  295,  295,  305,
 /*   730 */   310,  311,  317,  318,  316,  315,  312,  313,  314,  302,
 /*   740 */   301,   51,   45,   46,   43,   28,   35,  306,  303,   41,
 /*   750 */    42,  304,  248,   37,   17,  295,  295,  295,  295,  295,
 /*   760 */   295,  295,  295,  295,  295,  295,  305,  310,  311,  317,
 /*   770 */   318,  316,  315,  312,  313,  314,  302,  301,  146,  295,
 /*   780 */     8,  295,  119,   91,    6,  103,  295,  295,   12,  295,
 /*   790 */   130,  266,  263,  153,  219,  288,  201,  206,  218,   39,
 /*   800 */    23,  221,  277,   50,  295,   22,  295,  295,  295,   25,
 /*   810 */   295,  295,    8,  278,  132,  186,    6,  103,   49,   48,
 /*   820 */   220,  264,  130,  113,    1,  157,  219,  289,  201,  127,
 /*   830 */   218,  295,   23,  295,  277,   50,  295,   18,  295,   94,
 /*   840 */   295,  295,  278,  253,    8,  278,  132,  223,    6,  103,
 /*   850 */    49,   48,  220,  264,  130,  113,    1,  239,  219,   96,
 /*   860 */   201,   99,  218,  295,   23,  295,  295,   50,  295,  295,
 /*   870 */   295,   94,  295,  263,  295,  263,    8,  295,  132,  183,
 /*   880 */     6,  103,   49,   48,  220,  264,  130,  113,    1,  150,
 /*   890 */   219,  295,  201,  295,  192,  295,   23,  295,  277,   50,
 /*   900 */   295,  295,  295,   94,  295,  295,  295,  295,    8,  278,
 /*   910 */   133,  223,    6,  103,   49,   48,  220,  264,  130,  113,
 /*   920 */     1,  154,  219,  295,  201,  295,  218,  295,   31,  295,
 /*   930 */   277,   50,  295,  295,  295,   94,  295,  295,  295,  295,
 /*   940 */     8,  278,  132,  182,    6,  103,   49,   48,  220,  264,
 /*   950 */   130,  113,    1,  295,  219,  295,  201,  295,  218,  295,
 /*   960 */    23,  295,  295,   50,  295,  295,  295,   94,  295,  295,
 /*   970 */   295,  295,    8,  295,  132,  187,    6,  103,   49,   48,
 /*   980 */   220,  264,  130,  113,    1,  295,  219,  295,  201,  295,
 /*   990 */   218,  295,   23,  295,  295,   50,  295,  295,  295,   94,
 /*  1000 */   295,  295,  295,  295,    8,  295,  125,  223,    6,  103,
 /*  1010 */    49,   48,  220,  264,  130,  113,    1,  295,  219,  295,
 /*  1020 */   201,  295,  218,  295,    4,  295,  295,   50,  295,  295,
 /*  1030 */   295,   94,  295,  295,  295,  295,    8,  295,  118,  223,
 /*  1040 */     6,  103,   49,   48,  220,  264,  130,  113,    1,  295,
 /*  1050 */   219,  295,  201,  295,  218,  295,   23,  295,  295,   50,
 /*  1060 */   295,  295,  295,   94,  295,  295,  295,  295,    8,  295,
 /*  1070 */   133,  223,    6,  103,   49,   48,  220,  264,  130,  113,
 /*  1080 */     1,  295,  219,  295,  201,  295,  218,  295,   31,  295,
 /*  1090 */   295,   50,  295,  295,  295,   94,  295,  295,  295,  295,
 /*  1100 */     8,  295,  133,  217,    6,  103,   49,   48,  220,  264,
 /*  1110 */   130,  113,  295,  295,  219,  295,  201,  295,  218,  295,
 /*  1120 */    31,   97,  152,   50,  295,  295,  295,   94,  295,  295,
 /*  1130 */   295,  277,  420,  295,  295,   15,   34,  295,   49,   48,
 /*  1140 */   220,  264,  295,  113,  376,  181,  295,  420,  295,  420,
 /*  1150 */   420,  295,  420,  420,  216,  295,  295,  295,  420,   94,
 /*  1160 */   420,    5,  420,   12,  376,  295,  266,  295,  295,  295,
 /*  1170 */   376,  210,  295,    5,   39,  114,  222,  420,   76,  213,
 /*  1180 */    22,  295,  295,  295,  295,  295,  420,  159,  251,  290,
 /*  1190 */   295,  210,  250,  262,  111,  122,  295,  295,   57,  213,
 /*  1200 */   124,  295,  193,  294,  295,  173,  168,  145,  251,  290,
 /*  1210 */   295,  295,  250,  262,  111,  277,  295,  295,  210,   15,
 /*  1220 */    34,  295,  122,  295,  295,   74,  213,  295,  295,  181,
 /*  1230 */   295,  295,  295,  295,  159,  251,  290,  295,  295,  250,
 /*  1240 */   262,  111,  295,  295,  295,  295,  191,  173,  161,  295,
 /*  1250 */   295,  210,  295,  295,  295,  122,  295,  277,   74,  213,
 /*  1260 */   295,   15,   34,  295,  295,  295,  295,  159,  251,  290,
 /*  1270 */   295,  181,  250,  262,  111,  295,  295,  295,  210,  197,
 /*  1280 */   295,  295,  122,  295,  295,   60,  213,  198,  295,  295,
 /*  1290 */   295,  295,  295,  295,  145,  251,  290,  295,  210,  250,
 /*  1300 */   262,  111,  114,  295,  295,   76,  213,  295,  295,  295,
 /*  1310 */   295,  295,  295,  295,  159,  251,  290,  295,  295,  250,
 /*  1320 */   262,  111,  295,  295,  295,  295,  173,  160,  295,  295,
 /*  1330 */   297,  210,  295,  295,  295,  122,  277,  295,   52,  213,
 /*  1340 */    15,   34,  295,  295,  295,  295,  295,  159,  251,  290,
 /*  1350 */   181,  295,  250,  262,  111,  295,  295,  295,  210,  295,
 /*  1360 */   295,  295,  122,  295,  295,   80,  213,  295,  295,  295,
 /*  1370 */   295,  295,  295,  295,  159,  251,  290,  295,  210,  250,
 /*  1380 */   262,  111,  122,  295,  295,   62,  213,  295,  295,  295,
 /*  1390 */   295,  295,  295,  295,  159,  251,  290,  295,  295,  250,
 /*  1400 */   262,  111,  295,  295,  295,  173,  155,  295,  295,  295,
 /*  1410 */   210,  295,  295,  295,  112,  277,  295,   55,  213,   15,
 /*  1420 */    34,  295,  295,  295,  295,  295,  159,  251,  290,  181,
 /*  1430 */   295,  250,  262,  111,  210,  295,  295,  295,  122,  295,
 /*  1440 */   295,   88,  213,  295,  295,  295,  295,  295,  295,  295,
 /*  1450 */   159,  251,  290,  295,  210,  250,  262,  111,  122,  295,
 /*  1460 */   295,   58,  213,  295,  295,  295,  295,  295,  295,  295,
 /*  1470 */   159,  251,  290,  295,  210,  250,  262,  111,  122,  295,
 /*  1480 */   295,   78,  213,  295,  295,  295,  295,  295,  295,  295,
 /*  1490 */   159,  251,  290,  295,  210,  250,  262,  111,  122,  295,
 /*  1500 */   295,   67,  213,  295,  295,  295,  295,  295,  295,  295,
 /*  1510 */   159,  251,  290,  295,  210,  250,  262,  111,  122,  295,
 /*  1520 */   295,   61,  213,  295,  295,  295,  295,  295,  295,  295,
 /*  1530 */   159,  251,  290,  295,  210,  250,  262,  111,  122,  295,
 /*  1540 */   295,   64,  213,  295,  295,  295,  295,  295,  295,  295,
 /*  1550 */   159,  251,  290,  295,  210,  250,  262,  111,  122,  295,
 /*  1560 */   295,   89,  213,  295,  295,  295,  295,  295,  295,  295,
 /*  1570 */   159,  251,  290,  295,  210,  250,  262,  111,  122,  295,
 /*  1580 */   295,   75,  213,  295,  295,  295,  295,  295,  295,  295,
 /*  1590 */   159,  251,  290,  295,  210,  250,  262,  111,  122,  295,
 /*  1600 */   295,   81,  213,  295,  295,  295,  295,  295,  295,  295,
 /*  1610 */   159,  251,  290,  295,  210,  250,  262,  111,  122,  295,
 /*  1620 */   295,   69,  213,  295,  295,  295,  295,  295,  295,  295,
 /*  1630 */   159,  251,  290,  295,  210,  250,  262,  111,   92,  295,
 /*  1640 */   295,   54,  194,  295,  295,  295,  295,  295,  295,  295,
 /*  1650 */   159,  251,  290,  295,  210,  250,  262,  111,  122,  295,
 /*  1660 */   295,   65,  213,  295,  295,  295,  295,  295,  295,  295,
 /*  1670 */   159,  251,  290,  295,  210,  250,  262,  111,   92,  295,
 /*  1680 */   295,   53,  185,  295,  295,  295,  295,  295,  295,  295,
 /*  1690 */   159,  251,  290,  295,  210,  250,  262,  111,  122,  295,
 /*  1700 */   295,   66,  213,  295,  295,  295,  295,  295,  295,  295,
 /*  1710 */   159,  251,  290,  295,  210,  250,  262,  111,  122,  295,
 /*  1720 */   295,   79,  213,  295,  295,  295,  295,  295,  295,  295,
 /*  1730 */   159,  251,  290,  295,  210,  250,  262,  111,  122,  295,
 /*  1740 */   295,   84,  213,  295,  295,  295,  295,  295,  295,  295,
 /*  1750 */   159,  251,  290,  295,  210,  250,  262,  111,  122,  295,
 /*  1760 */   295,   82,  213,  295,  295,  295,  295,  295,  295,  295,
 /*  1770 */   159,  251,  290,  295,  210,  250,  262,  111,  122,  295,
 /*  1780 */   295,   83,  213,  295,  295,  295,  295,  295,  295,  295,
 /*  1790 */   159,  251,  290,  295,  210,  250,  262,  111,  122,  295,
 /*  1800 */   295,   72,  213,  295,  295,  295,  295,  295,  295,  295,
 /*  1810 */   159,  251,  290,  295,  210,  250,  262,  111,  122,  295,
 /*  1820 */   295,   73,  213,  295,  295,  295,  295,  295,  295,  295,
 /*  1830 */   159,  251,  290,  295,  210,  250,  262,  111,  105,  295,
 /*  1840 */   295,   63,  213,  295,  295,  295,  295,  295,  295,  295,
 /*  1850 */   159,  251,  290,  295,  210,  250,  262,  111,  122,  295,
 /*  1860 */   295,   86,  213,  295,  295,  295,  295,  295,  295,  295,
 /*  1870 */   159,  251,  290,  295,  210,  250,  262,  111,  122,  295,
 /*  1880 */   295,   77,  213,  295,  295,  295,  295,  295,  295,  295,
 /*  1890 */   159,  251,  290,  295,  210,  250,  262,  111,  122,  295,
 /*  1900 */   295,   71,  213,  295,  295,  295,  295,  295,  295,  295,
 /*  1910 */   159,  251,  290,  295,  210,  250,  262,  111,  122,  295,
 /*  1920 */   295,   56,  213,  295,  295,  295,  295,  295,  295,  295,
 /*  1930 */   159,  251,  290,  295,  210,  250,  262,  111,  107,  295,
 /*  1940 */   295,   87,  213,  295,  295,  295,  295,  295,  295,  295,
 /*  1950 */   159,  251,  290,  295,  210,  250,  262,  111,  122,  295,
 /*  1960 */   295,   60,  213,  295,  295,  295,  295,  295,  295,  295,
 /*  1970 */   159,  251,  290,  295,  210,  250,  262,  111,  122,  295,
 /*  1980 */   295,   70,  213,  295,  295,  295,  295,  295,  295,  295,
 /*  1990 */   159,  251,  290,  295,  210,  250,  262,  111,  122,  295,
 /*  2000 */   295,   59,  213,  295,  295,  295,  295,  295,  295,  295,
 /*  2010 */   159,  251,  290,  295,  210,  250,  262,  111,  122,  295,
 /*  2020 */   295,   68,  213,  295,  295,  295,  295,  295,  295,  295,
 /*  2030 */   159,  251,  290,  295,  210,  250,  262,  111,  106,  295,
 /*  2040 */   295,   85,  213,  295,  295,  295,  295,  295,  295,  295,
 /*  2050 */   159,  251,  290,  295,  295,  250,  262,  111,  299,  295,
 /*  2060 */   295,  295,  295,  295,  295,   10,  295,  295,  295,    6,
 /*  2070 */   103,  295,  210,  295,  295,  130,  137,  295,  295,  219,
 /*  2080 */   213,  201,  295,  295,  295,  295,  295,  295,  159,  295,
 /*  2090 */   309,  295,  210,  250,  262,  111,  141,  295,  295,  295,
 /*  2100 */   213,  295,  295,  295,  295,  295,  295,  295,  159,  295,
 /*  2110 */   295,  295,  210,  250,  262,  111,  140,  295,  295,  295,
 /*  2120 */   213,  295,  210,  295,  285,  292,  135,  295,  159,  295,
 /*  2130 */   213,  295,  210,  250,  262,  111,  139,  295,  159,  295,
 /*  2140 */   213,  295,  295,  250,  262,  111,  295,  295,  159,  295,
 /*  2150 */   295,  295,  210,  250,  262,  111,  138,  295,  295,  295,
 /*  2160 */   213,  210,  295,  295,  295,  134,  295,  295,  159,  213,
 /*  2170 */   295,  295,  210,  250,  262,  111,  136,  159,  295,  295,
 /*  2180 */   213,  295,  250,  262,  111,  295,  295,  295,  159,  295,
 /*  2190 */   295,  295,  295,  250,  262,  111,
    );
    static public $yy_lookahead = array(
 /*     0 */     1,   84,   13,    4,   15,   16,    5,    6,    7,    8,
 /*    10 */     9,   10,   91,   53,   13,   98,   99,   57,   17,   18,
 /*    20 */    31,   75,   76,   24,   23,  108,  105,   53,   27,   30,
 /*    30 */    29,   57,    1,   59,   35,   36,   37,   38,   39,   40,
 /*    40 */    41,   42,   43,   44,   45,   46,   47,   48,   13,   14,
 /*    50 */    53,   16,   33,   56,   57,   20,   59,   26,    4,   60,
 /*    60 */    61,   62,   63,   64,   65,   66,   67,   68,   69,   70,
 /*    70 */    71,    1,   53,   13,   14,   79,   16,   58,    6,   83,
 /*    80 */    49,   13,   14,   87,   16,   13,   10,   11,   12,   17,
 /*    90 */    18,   95,   22,   97,   24,   23,  100,  101,  102,   27,
 /*   100 */    30,   29,    1,   32,   14,   35,   36,   37,   38,   39,
 /*   110 */    40,   41,   42,   43,   44,   45,   46,   47,   48,    1,
 /*   120 */    78,   79,   80,   55,   77,   78,   79,   80,  108,   58,
 /*   130 */    60,   61,   62,   63,   64,   65,   66,   67,   68,   69,
 /*   140 */    70,   71,   24,   53,   72,   73,    1,   57,   30,    4,
 /*   150 */    49,   15,   16,   35,   36,   37,   38,   39,   40,   41,
 /*   160 */    42,   43,   44,   45,   46,   47,   48,    1,   85,   24,
 /*   170 */    25,   13,   93,   15,   16,   30,   91,   94,   60,   61,
 /*   180 */    62,   63,   64,   65,   66,   67,   68,   69,   70,   71,
 /*   190 */   105,   25,   56,    1,   49,    1,    4,   13,    4,   84,
 /*   200 */    16,   35,   36,   37,   38,   39,   40,   41,   42,   43,
 /*   210 */    44,   45,   46,   47,   48,    1,   24,    1,   24,   32,
 /*   220 */     4,   34,   30,  108,   30,   33,   60,   61,   62,   63,
 /*   230 */    64,   65,   66,   67,   68,   69,   70,   71,   79,   55,
 /*   240 */    24,   93,    1,   49,   13,    4,   30,   16,   34,   35,
 /*   250 */    36,   37,   38,   39,   40,   41,   42,   43,   44,   45,
 /*   260 */    46,   47,   48,    1,    1,   49,    4,    4,   13,    1,
 /*   270 */    84,  112,  113,   93,   60,   61,   62,   63,   64,   65,
 /*   280 */    66,   67,   68,   69,   70,   71,   55,   24,   33,   32,
 /*   290 */    49,   34,    1,   30,  108,  109,   84,   35,   36,   37,
 /*   300 */    38,   39,   40,   41,   42,   43,   44,   45,   46,   47,
 /*   310 */    48,    1,   49,    1,    4,   24,    4,   32,   34,   34,
 /*   320 */   108,   30,   60,   61,   62,   63,   64,   65,   66,   67,
 /*   330 */    68,   69,   70,   71,   88,   89,   24,   24,   14,  103,
 /*   340 */   104,   95,   30,   30,   20,   35,   36,   37,   38,   39,
 /*   350 */    40,   41,   42,   43,   44,   45,   46,   47,   48,   84,
 /*   360 */    85,   26,   80,   13,   82,   14,   16,   33,   33,   94,
 /*   370 */    60,   61,   62,   63,   64,   65,   66,   67,   68,   69,
 /*   380 */    70,   71,    1,  108,   16,   79,   84,   85,   13,   83,
 /*   390 */    95,   16,   86,   87,   31,   91,   94,  102,   19,   24,
 /*   400 */    19,   95,   96,   97,   53,   30,  100,  101,  102,  105,
 /*   410 */   108,   32,   13,  107,   53,   16,   35,   36,   37,   38,
 /*   420 */    39,   40,   41,   42,   43,   44,   45,   46,   47,   48,
 /*   430 */     1,   85,   13,    4,   33,   16,   13,   15,   50,   16,
 /*   440 */    94,   60,   61,   62,   63,   64,   65,   66,   67,   68,
 /*   450 */    69,   70,   71,   13,  108,   33,   16,   24,    1,    2,
 /*   460 */    15,   20,   16,   30,   35,   36,   37,   38,   39,   40,
 /*   470 */    41,   42,   43,   44,   45,   46,   47,   48,    1,   85,
 /*   480 */    88,    4,   16,   15,   15,   15,   15,   95,   94,   60,
 /*   490 */    61,   62,   63,   64,   65,   66,   67,   68,   69,   70,
 /*   500 */    71,   55,  108,   21,   88,   16,   16,   56,    4,    2,
 /*   510 */    16,   95,   35,   36,   37,   38,   39,   40,   41,   42,
 /*   520 */    43,   44,   45,   46,   47,   48,    1,   85,   16,   16,
 /*   530 */     2,   31,   16,    4,   14,   91,   94,   60,   61,   62,
 /*   540 */    63,   64,   65,   66,   67,   68,   69,   70,   71,  105,
 /*   550 */   108,  103,   94,   58,    3,  105,  103,   24,  106,   34,
 /*   560 */    35,   36,   37,   38,   39,   40,   41,   42,   43,   44,
 /*   570 */    45,   46,   47,   48,    1,    2,  109,   90,   90,   11,
 /*   580 */     2,  114,   81,   91,   91,   60,   61,   62,   63,   64,
 /*   590 */    65,   66,   67,   68,   69,   70,   71,  105,  105,  114,
 /*   600 */   114,  114,  114,  114,  114,  114,  114,  114,   35,   36,
 /*   610 */    37,   38,   39,   40,   41,   42,   43,   44,   45,   46,
 /*   620 */    47,   48,    1,   85,  114,  114,  114,  114,  114,  114,
 /*   630 */   114,   91,   94,   60,   61,   62,   63,   64,   65,   66,
 /*   640 */    67,   68,   69,   70,   71,  105,   79,  114,  114,  114,
 /*   650 */   114,  114,  114,  114,  114,  114,   35,   36,   37,   38,
 /*   660 */    39,   40,   41,   42,   43,   44,   45,   46,   47,   48,
 /*   670 */     1,  114,  114,  114,  114,  114,  114,  114,  114,   58,
 /*   680 */   113,   60,   61,   62,   63,   64,   65,   66,   67,   68,
 /*   690 */    69,   70,   71,  114,   85,  114,  114,  114,  114,  114,
 /*   700 */   114,  114,  114,   94,   35,   36,   37,   38,   39,   40,
 /*   710 */    41,   42,   43,   44,   45,   46,   47,   48,  114,  114,
 /*   720 */   114,  114,  114,  114,  114,  114,  114,  114,  114,   60,
 /*   730 */    61,   62,   63,   64,   65,   66,   67,   68,   69,   70,
 /*   740 */    71,   35,   36,   37,   38,   39,   40,   41,   42,   43,
 /*   750 */    44,   45,   46,   47,   48,  114,  114,  114,  114,  114,
 /*   760 */   114,  114,  114,  114,  114,  114,   60,   61,   62,   63,
 /*   770 */    64,   65,   66,   67,   68,   69,   70,   71,   91,  114,
 /*   780 */    13,  114,   15,   16,   17,   18,  114,  114,   13,  114,
 /*   790 */    23,   16,  105,   85,   27,   28,   29,    2,   31,   24,
 /*   800 */    33,   26,   94,   36,  114,   30,  114,  114,  114,   14,
 /*   810 */   114,  114,   13,  105,   15,   16,   17,   18,   51,   52,
 /*   820 */    53,   54,   23,   56,   57,   85,   27,    4,   29,   92,
 /*   830 */    31,  114,   33,  114,   94,   36,  114,   14,  114,   72,
 /*   840 */   114,  114,  105,   20,   13,  105,   15,   16,   17,   18,
 /*   850 */    51,   52,   53,   54,   23,   56,   57,   58,   27,   91,
 /*   860 */    29,   91,   31,  114,   33,  114,  114,   36,  114,  114,
 /*   870 */   114,   72,  114,  105,  114,  105,   13,  114,   15,   16,
 /*   880 */    17,   18,   51,   52,   53,   54,   23,   56,   57,   85,
 /*   890 */    27,  114,   29,  114,   31,  114,   33,  114,   94,   36,
 /*   900 */   114,  114,  114,   72,  114,  114,  114,  114,   13,  105,
 /*   910 */    15,   16,   17,   18,   51,   52,   53,   54,   23,   56,
 /*   920 */    57,   85,   27,  114,   29,  114,   31,  114,   33,  114,
 /*   930 */    94,   36,  114,  114,  114,   72,  114,  114,  114,  114,
 /*   940 */    13,  105,   15,   16,   17,   18,   51,   52,   53,   54,
 /*   950 */    23,   56,   57,  114,   27,  114,   29,  114,   31,  114,
 /*   960 */    33,  114,  114,   36,  114,  114,  114,   72,  114,  114,
 /*   970 */   114,  114,   13,  114,   15,   16,   17,   18,   51,   52,
 /*   980 */    53,   54,   23,   56,   57,  114,   27,  114,   29,  114,
 /*   990 */    31,  114,   33,  114,  114,   36,  114,  114,  114,   72,
 /*  1000 */   114,  114,  114,  114,   13,  114,   15,   16,   17,   18,
 /*  1010 */    51,   52,   53,   54,   23,   56,   57,  114,   27,  114,
 /*  1020 */    29,  114,   31,  114,   33,  114,  114,   36,  114,  114,
 /*  1030 */   114,   72,  114,  114,  114,  114,   13,  114,   15,   16,
 /*  1040 */    17,   18,   51,   52,   53,   54,   23,   56,   57,  114,
 /*  1050 */    27,  114,   29,  114,   31,  114,   33,  114,  114,   36,
 /*  1060 */   114,  114,  114,   72,  114,  114,  114,  114,   13,  114,
 /*  1070 */    15,   16,   17,   18,   51,   52,   53,   54,   23,   56,
 /*  1080 */    57,  114,   27,  114,   29,  114,   31,  114,   33,  114,
 /*  1090 */   114,   36,  114,  114,  114,   72,  114,  114,  114,  114,
 /*  1100 */    13,  114,   15,   16,   17,   18,   51,   52,   53,   54,
 /*  1110 */    23,   56,  114,  114,   27,  114,   29,  114,   31,  114,
 /*  1120 */    33,   84,   85,   36,  114,  114,  114,   72,  114,  114,
 /*  1130 */   114,   94,    4,  114,  114,   98,   99,  114,   51,   52,
 /*  1140 */    53,   54,  114,   56,    4,  108,  114,   19,  114,   21,
 /*  1150 */    22,  114,   24,   25,   14,  114,  114,  114,   30,   72,
 /*  1160 */    32,   33,   34,   13,   24,  114,   16,  114,  114,  114,
 /*  1170 */    30,   79,  114,   33,   24,   83,   26,   49,   86,   87,
 /*  1180 */    30,  114,  114,  114,  114,  114,   58,   95,   96,   97,
 /*  1190 */   114,   79,  100,  101,  102,   83,  114,  114,   86,   87,
 /*  1200 */    88,  114,  110,  111,  114,   84,   85,   95,   96,   97,
 /*  1210 */   114,  114,  100,  101,  102,   94,  114,  114,   79,   98,
 /*  1220 */    99,  114,   83,  114,  114,   86,   87,  114,  114,  108,
 /*  1230 */   114,  114,  114,  114,   95,   96,   97,  114,  114,  100,
 /*  1240 */   101,  102,  114,  114,  114,  114,  107,   84,   85,  114,
 /*  1250 */   114,   79,  114,  114,  114,   83,  114,   94,   86,   87,
 /*  1260 */   114,   98,   99,  114,  114,  114,  114,   95,   96,   97,
 /*  1270 */   114,  108,  100,  101,  102,  114,  114,  114,   79,  107,
 /*  1280 */   114,  114,   83,  114,  114,   86,   87,   88,  114,  114,
 /*  1290 */   114,  114,  114,  114,   95,   96,   97,  114,   79,  100,
 /*  1300 */   101,  102,   83,  114,  114,   86,   87,  114,  114,  114,
 /*  1310 */   114,  114,  114,  114,   95,   96,   97,  114,  114,  100,
 /*  1320 */   101,  102,  114,  114,  114,  114,   84,   85,  114,  114,
 /*  1330 */   111,   79,  114,  114,  114,   83,   94,  114,   86,   87,
 /*  1340 */    98,   99,  114,  114,  114,  114,  114,   95,   96,   97,
 /*  1350 */   108,  114,  100,  101,  102,  114,  114,  114,   79,  114,
 /*  1360 */   114,  114,   83,  114,  114,   86,   87,  114,  114,  114,
 /*  1370 */   114,  114,  114,  114,   95,   96,   97,  114,   79,  100,
 /*  1380 */   101,  102,   83,  114,  114,   86,   87,  114,  114,  114,
 /*  1390 */   114,  114,  114,  114,   95,   96,   97,  114,  114,  100,
 /*  1400 */   101,  102,  114,  114,  114,   84,   85,  114,  114,  114,
 /*  1410 */    79,  114,  114,  114,   83,   94,  114,   86,   87,   98,
 /*  1420 */    99,  114,  114,  114,  114,  114,   95,   96,   97,  108,
 /*  1430 */   114,  100,  101,  102,   79,  114,  114,  114,   83,  114,
 /*  1440 */   114,   86,   87,  114,  114,  114,  114,  114,  114,  114,
 /*  1450 */    95,   96,   97,  114,   79,  100,  101,  102,   83,  114,
 /*  1460 */   114,   86,   87,  114,  114,  114,  114,  114,  114,  114,
 /*  1470 */    95,   96,   97,  114,   79,  100,  101,  102,   83,  114,
 /*  1480 */   114,   86,   87,  114,  114,  114,  114,  114,  114,  114,
 /*  1490 */    95,   96,   97,  114,   79,  100,  101,  102,   83,  114,
 /*  1500 */   114,   86,   87,  114,  114,  114,  114,  114,  114,  114,
 /*  1510 */    95,   96,   97,  114,   79,  100,  101,  102,   83,  114,
 /*  1520 */   114,   86,   87,  114,  114,  114,  114,  114,  114,  114,
 /*  1530 */    95,   96,   97,  114,   79,  100,  101,  102,   83,  114,
 /*  1540 */   114,   86,   87,  114,  114,  114,  114,  114,  114,  114,
 /*  1550 */    95,   96,   97,  114,   79,  100,  101,  102,   83,  114,
 /*  1560 */   114,   86,   87,  114,  114,  114,  114,  114,  114,  114,
 /*  1570 */    95,   96,   97,  114,   79,  100,  101,  102,   83,  114,
 /*  1580 */   114,   86,   87,  114,  114,  114,  114,  114,  114,  114,
 /*  1590 */    95,   96,   97,  114,   79,  100,  101,  102,   83,  114,
 /*  1600 */   114,   86,   87,  114,  114,  114,  114,  114,  114,  114,
 /*  1610 */    95,   96,   97,  114,   79,  100,  101,  102,   83,  114,
 /*  1620 */   114,   86,   87,  114,  114,  114,  114,  114,  114,  114,
 /*  1630 */    95,   96,   97,  114,   79,  100,  101,  102,   83,  114,
 /*  1640 */   114,   86,   87,  114,  114,  114,  114,  114,  114,  114,
 /*  1650 */    95,   96,   97,  114,   79,  100,  101,  102,   83,  114,
 /*  1660 */   114,   86,   87,  114,  114,  114,  114,  114,  114,  114,
 /*  1670 */    95,   96,   97,  114,   79,  100,  101,  102,   83,  114,
 /*  1680 */   114,   86,   87,  114,  114,  114,  114,  114,  114,  114,
 /*  1690 */    95,   96,   97,  114,   79,  100,  101,  102,   83,  114,
 /*  1700 */   114,   86,   87,  114,  114,  114,  114,  114,  114,  114,
 /*  1710 */    95,   96,   97,  114,   79,  100,  101,  102,   83,  114,
 /*  1720 */   114,   86,   87,  114,  114,  114,  114,  114,  114,  114,
 /*  1730 */    95,   96,   97,  114,   79,  100,  101,  102,   83,  114,
 /*  1740 */   114,   86,   87,  114,  114,  114,  114,  114,  114,  114,
 /*  1750 */    95,   96,   97,  114,   79,  100,  101,  102,   83,  114,
 /*  1760 */   114,   86,   87,  114,  114,  114,  114,  114,  114,  114,
 /*  1770 */    95,   96,   97,  114,   79,  100,  101,  102,   83,  114,
 /*  1780 */   114,   86,   87,  114,  114,  114,  114,  114,  114,  114,
 /*  1790 */    95,   96,   97,  114,   79,  100,  101,  102,   83,  114,
 /*  1800 */   114,   86,   87,  114,  114,  114,  114,  114,  114,  114,
 /*  1810 */    95,   96,   97,  114,   79,  100,  101,  102,   83,  114,
 /*  1820 */   114,   86,   87,  114,  114,  114,  114,  114,  114,  114,
 /*  1830 */    95,   96,   97,  114,   79,  100,  101,  102,   83,  114,
 /*  1840 */   114,   86,   87,  114,  114,  114,  114,  114,  114,  114,
 /*  1850 */    95,   96,   97,  114,   79,  100,  101,  102,   83,  114,
 /*  1860 */   114,   86,   87,  114,  114,  114,  114,  114,  114,  114,
 /*  1870 */    95,   96,   97,  114,   79,  100,  101,  102,   83,  114,
 /*  1880 */   114,   86,   87,  114,  114,  114,  114,  114,  114,  114,
 /*  1890 */    95,   96,   97,  114,   79,  100,  101,  102,   83,  114,
 /*  1900 */   114,   86,   87,  114,  114,  114,  114,  114,  114,  114,
 /*  1910 */    95,   96,   97,  114,   79,  100,  101,  102,   83,  114,
 /*  1920 */   114,   86,   87,  114,  114,  114,  114,  114,  114,  114,
 /*  1930 */    95,   96,   97,  114,   79,  100,  101,  102,   83,  114,
 /*  1940 */   114,   86,   87,  114,  114,  114,  114,  114,  114,  114,
 /*  1950 */    95,   96,   97,  114,   79,  100,  101,  102,   83,  114,
 /*  1960 */   114,   86,   87,  114,  114,  114,  114,  114,  114,  114,
 /*  1970 */    95,   96,   97,  114,   79,  100,  101,  102,   83,  114,
 /*  1980 */   114,   86,   87,  114,  114,  114,  114,  114,  114,  114,
 /*  1990 */    95,   96,   97,  114,   79,  100,  101,  102,   83,  114,
 /*  2000 */   114,   86,   87,  114,  114,  114,  114,  114,  114,  114,
 /*  2010 */    95,   96,   97,  114,   79,  100,  101,  102,   83,  114,
 /*  2020 */   114,   86,   87,  114,  114,  114,  114,  114,  114,  114,
 /*  2030 */    95,   96,   97,  114,   79,  100,  101,  102,   83,  114,
 /*  2040 */   114,   86,   87,  114,  114,  114,  114,  114,  114,  114,
 /*  2050 */    95,   96,   97,  114,  114,  100,  101,  102,    6,  114,
 /*  2060 */   114,  114,  114,  114,  114,   13,  114,  114,  114,   17,
 /*  2070 */    18,  114,   79,  114,  114,   23,   83,  114,  114,   27,
 /*  2080 */    87,   29,  114,  114,  114,  114,  114,  114,   95,  114,
 /*  2090 */    97,  114,   79,  100,  101,  102,   83,  114,  114,  114,
 /*  2100 */    87,  114,  114,  114,  114,  114,  114,  114,   95,  114,
 /*  2110 */   114,  114,   79,  100,  101,  102,   83,  114,  114,  114,
 /*  2120 */    87,  114,   79,  114,   72,   73,   83,  114,   95,  114,
 /*  2130 */    87,  114,   79,  100,  101,  102,   83,  114,   95,  114,
 /*  2140 */    87,  114,  114,  100,  101,  102,  114,  114,   95,  114,
 /*  2150 */   114,  114,   79,  100,  101,  102,   83,  114,  114,  114,
 /*  2160 */    87,   79,  114,  114,  114,   83,  114,  114,   95,   87,
 /*  2170 */   114,  114,   79,  100,  101,  102,   83,   95,  114,  114,
 /*  2180 */    87,  114,  100,  101,  102,  114,  114,  114,   95,  114,
 /*  2190 */   114,  114,  114,  100,  101,  102,
);
    const YY_SHIFT_USE_DFLT = -41;
    const YY_SHIFT_MAX = 228;
    static public $yy_shift_ofst = array(
 /*     0 */   551,  959,  959,  831,  991,  831,  991,  831,  767,  799,
 /*    10 */   767,  831,  831,  831,  831,  831,  927,  831,  831,  863,
 /*    20 */   831,  831,  831,  831,  831,  831, 1023,  831,  831,  831,
 /*    30 */   831,  831,  831,  831,  831,  831,  831,  831,  831,  863,
 /*    40 */   831,  831,  831,  895,  895, 1055, 1055, 1087, 1055, 1055,
 /*    50 */  1055, 1055,   70,   -1,  118,  118,  118,  118,  381,  477,
 /*    60 */   525,  573,  310,  166,  214,  262,  621,  429,  669,  669,
 /*    70 */   669,  669,  669,  669,  669,  669,  669,  669,  669,  669,
 /*    80 */   669,  669,  669,  669,  669,  669,  669,  669,  706,  706,
 /*    90 */     1,  192,  216,    1, 2052,  775, 1150,  312,  375,  375,
 /*   100 */   312,  291,  457,  422,   72,  145,  194,  263,   -3,   35,
 /*   110 */    76,  -26,  241,  136,   31,  419,  350,  350,  423,  440,
 /*   120 */   350,  422,  101,  422,  313,  399,  350,  313,  350,  350,
 /*   130 */   433,  350,  399,  350,  268,  268,  268,  268,  268,  268,
 /*   140 */   268,  268,  268,  -41,   68,   90,   60,  184,  231,  158,
 /*   150 */   313,  -40,  313,  313,  313,  313,  313,  313,  313,  -40,
 /*   160 */   313,  313,  313,  313,  313,  313,  350,  313,  313,  313,
 /*   170 */   268,  268,  533,  268,  568,  533,  578,  578,  268,  -41,
 /*   180 */   -41,  -41, 1128, 1140,  -11,  823,   19,  335,  255,  379,
 /*   190 */   795,  187,  351,   71,  324,  446,  257,  285,  284,   54,
 /*   200 */   495,  512,  504,  494,  490,  451,  513,  334,  520,  516,
 /*   210 */   529,  528,  500,  441,  482,  445,  466,  401,  361,  368,
 /*   220 */   363,  468,  469,  401,  471,  388,  470,  489,  507,
);
    const YY_REDUCE_USE_DFLT = -84;
    const YY_REDUCE_MAX = 181;
    static public $yy_reduce_ofst = array(
 /*     0 */   -54, 1092, 1219, 1172, 1199, 1139, 1112,  306, 1555, 1615,
 /*    10 */  1595, 1635, 1299, 1252, 1375, 1655, 1355, 1895, 1695, 1755,
 /*    20 */  1775, 1835, 1855, 1875, 1675, 1331, 1435, 1495, 1935, 1515,
 /*    30 */  1475, 1455, 1575, 1279, 1395, 1535, 1795, 1815, 1415, 1955,
 /*    40 */  1915, 1715, 1735, 1993,   -4, 2013, 2073, 2093, 2053, 2082,
 /*    50 */  2043, 2033, 1242, 1037, 1037, 1121, 1321, 1163,  -83,  -83,
 /*    60 */   -83,  -83,  -83,  -83,  -83,  -83,  -83,  -83,  -83,  -83,
 /*    70 */   -83,  -83,  -83,  -83,  -83,  -83,  -83,  -83,  -83,  -83,
 /*    80 */   -83,  -83,  -83,  -83,  -83,  -83,  -83,  -83,  -83,  -83,
 /*    90 */    47,  302,  275,   42,  159,  804,  836,  394,  708,  740,
 /*   100 */   346,  442,  186,  246,  567,  212,  212,  212,  236,  737,
 /*   110 */   282,  236,  212,  295,  212,  -79,  493,  492,  444,  444,
 /*   120 */   304,  416,  212,  392,   83,   85,  540,  609,  770,  768,
 /*   130 */   538,  687,  444,  444,  212,  212,  212,  212,  212,  212,
 /*   140 */   212,  212,  115,  212,  450,  448,  450,  450,  450,  452,
 /*   150 */   458,  453,  458,  458,  458,  458,  458,  458,  458,  448,
 /*   160 */   458,  458,  458,  458,  458,  458,  450,  458,  458,  458,
 /*   170 */    20,   20,  487,   20,  501,  488,  467,  467,   20,   79,
 /*   180 */   180,  148,
);
    static public $yyExpectedTokens = array(
        /* 0 */ array(3, ),
        /* 1 */ array(13, 15, 16, 17, 18, 23, 27, 29, 31, 33, 36, 51, 52, 53, 54, 56, 57, 72, ),
        /* 2 */ array(13, 15, 16, 17, 18, 23, 27, 29, 31, 33, 36, 51, 52, 53, 54, 56, 57, 72, ),
        /* 3 */ array(13, 15, 16, 17, 18, 23, 27, 29, 31, 33, 36, 51, 52, 53, 54, 56, 57, 72, ),
        /* 4 */ array(13, 15, 16, 17, 18, 23, 27, 29, 31, 33, 36, 51, 52, 53, 54, 56, 57, 72, ),
        /* 5 */ array(13, 15, 16, 17, 18, 23, 27, 29, 31, 33, 36, 51, 52, 53, 54, 56, 57, 72, ),
        /* 6 */ array(13, 15, 16, 17, 18, 23, 27, 29, 31, 33, 36, 51, 52, 53, 54, 56, 57, 72, ),
        /* 7 */ array(13, 15, 16, 17, 18, 23, 27, 29, 31, 33, 36, 51, 52, 53, 54, 56, 57, 72, ),
        /* 8 */ array(13, 15, 16, 17, 18, 23, 27, 28, 29, 31, 33, 36, 51, 52, 53, 54, 56, 57, 72, ),
        /* 9 */ array(13, 15, 16, 17, 18, 23, 27, 29, 31, 33, 36, 51, 52, 53, 54, 56, 57, 58, 72, ),
        /* 10 */ array(13, 15, 16, 17, 18, 23, 27, 28, 29, 31, 33, 36, 51, 52, 53, 54, 56, 57, 72, ),
        /* 11 */ array(13, 15, 16, 17, 18, 23, 27, 29, 31, 33, 36, 51, 52, 53, 54, 56, 57, 72, ),
        /* 12 */ array(13, 15, 16, 17, 18, 23, 27, 29, 31, 33, 36, 51, 52, 53, 54, 56, 57, 72, ),
        /* 13 */ array(13, 15, 16, 17, 18, 23, 27, 29, 31, 33, 36, 51, 52, 53, 54, 56, 57, 72, ),
        /* 14 */ array(13, 15, 16, 17, 18, 23, 27, 29, 31, 33, 36, 51, 52, 53, 54, 56, 57, 72, ),
        /* 15 */ array(13, 15, 16, 17, 18, 23, 27, 29, 31, 33, 36, 51, 52, 53, 54, 56, 57, 72, ),
        /* 16 */ array(13, 15, 16, 17, 18, 23, 27, 29, 31, 33, 36, 51, 52, 53, 54, 56, 57, 72, ),
        /* 17 */ array(13, 15, 16, 17, 18, 23, 27, 29, 31, 33, 36, 51, 52, 53, 54, 56, 57, 72, ),
        /* 18 */ array(13, 15, 16, 17, 18, 23, 27, 29, 31, 33, 36, 51, 52, 53, 54, 56, 57, 72, ),
        /* 19 */ array(13, 15, 16, 17, 18, 23, 27, 29, 31, 33, 36, 51, 52, 53, 54, 56, 57, 72, ),
        /* 20 */ array(13, 15, 16, 17, 18, 23, 27, 29, 31, 33, 36, 51, 52, 53, 54, 56, 57, 72, ),
        /* 21 */ array(13, 15, 16, 17, 18, 23, 27, 29, 31, 33, 36, 51, 52, 53, 54, 56, 57, 72, ),
        /* 22 */ array(13, 15, 16, 17, 18, 23, 27, 29, 31, 33, 36, 51, 52, 53, 54, 56, 57, 72, ),
        /* 23 */ array(13, 15, 16, 17, 18, 23, 27, 29, 31, 33, 36, 51, 52, 53, 54, 56, 57, 72, ),
        /* 24 */ array(13, 15, 16, 17, 18, 23, 27, 29, 31, 33, 36, 51, 52, 53, 54, 56, 57, 72, ),
        /* 25 */ array(13, 15, 16, 17, 18, 23, 27, 29, 31, 33, 36, 51, 52, 53, 54, 56, 57, 72, ),
        /* 26 */ array(13, 15, 16, 17, 18, 23, 27, 29, 31, 33, 36, 51, 52, 53, 54, 56, 57, 72, ),
        /* 27 */ array(13, 15, 16, 17, 18, 23, 27, 29, 31, 33, 36, 51, 52, 53, 54, 56, 57, 72, ),
        /* 28 */ array(13, 15, 16, 17, 18, 23, 27, 29, 31, 33, 36, 51, 52, 53, 54, 56, 57, 72, ),
        /* 29 */ array(13, 15, 16, 17, 18, 23, 27, 29, 31, 33, 36, 51, 52, 53, 54, 56, 57, 72, ),
        /* 30 */ array(13, 15, 16, 17, 18, 23, 27, 29, 31, 33, 36, 51, 52, 53, 54, 56, 57, 72, ),
        /* 31 */ array(13, 15, 16, 17, 18, 23, 27, 29, 31, 33, 36, 51, 52, 53, 54, 56, 57, 72, ),
        /* 32 */ array(13, 15, 16, 17, 18, 23, 27, 29, 31, 33, 36, 51, 52, 53, 54, 56, 57, 72, ),
        /* 33 */ array(13, 15, 16, 17, 18, 23, 27, 29, 31, 33, 36, 51, 52, 53, 54, 56, 57, 72, ),
        /* 34 */ array(13, 15, 16, 17, 18, 23, 27, 29, 31, 33, 36, 51, 52, 53, 54, 56, 57, 72, ),
        /* 35 */ array(13, 15, 16, 17, 18, 23, 27, 29, 31, 33, 36, 51, 52, 53, 54, 56, 57, 72, ),
        /* 36 */ array(13, 15, 16, 17, 18, 23, 27, 29, 31, 33, 36, 51, 52, 53, 54, 56, 57, 72, ),
        /* 37 */ array(13, 15, 16, 17, 18, 23, 27, 29, 31, 33, 36, 51, 52, 53, 54, 56, 57, 72, ),
        /* 38 */ array(13, 15, 16, 17, 18, 23, 27, 29, 31, 33, 36, 51, 52, 53, 54, 56, 57, 72, ),
        /* 39 */ array(13, 15, 16, 17, 18, 23, 27, 29, 31, 33, 36, 51, 52, 53, 54, 56, 57, 72, ),
        /* 40 */ array(13, 15, 16, 17, 18, 23, 27, 29, 31, 33, 36, 51, 52, 53, 54, 56, 57, 72, ),
        /* 41 */ array(13, 15, 16, 17, 18, 23, 27, 29, 31, 33, 36, 51, 52, 53, 54, 56, 57, 72, ),
        /* 42 */ array(13, 15, 16, 17, 18, 23, 27, 29, 31, 33, 36, 51, 52, 53, 54, 56, 57, 72, ),
        /* 43 */ array(13, 15, 16, 17, 18, 23, 27, 29, 31, 33, 36, 51, 52, 53, 54, 56, 57, 72, ),
        /* 44 */ array(13, 15, 16, 17, 18, 23, 27, 29, 31, 33, 36, 51, 52, 53, 54, 56, 57, 72, ),
        /* 45 */ array(13, 15, 16, 17, 18, 23, 27, 29, 31, 33, 36, 51, 52, 53, 54, 56, 72, ),
        /* 46 */ array(13, 15, 16, 17, 18, 23, 27, 29, 31, 33, 36, 51, 52, 53, 54, 56, 72, ),
        /* 47 */ array(13, 15, 16, 17, 18, 23, 27, 29, 31, 33, 36, 51, 52, 53, 54, 56, 72, ),
        /* 48 */ array(13, 15, 16, 17, 18, 23, 27, 29, 31, 33, 36, 51, 52, 53, 54, 56, 72, ),
        /* 49 */ array(13, 15, 16, 17, 18, 23, 27, 29, 31, 33, 36, 51, 52, 53, 54, 56, 72, ),
        /* 50 */ array(13, 15, 16, 17, 18, 23, 27, 29, 31, 33, 36, 51, 52, 53, 54, 56, 72, ),
        /* 51 */ array(13, 15, 16, 17, 18, 23, 27, 29, 31, 33, 36, 51, 52, 53, 54, 56, 72, ),
        /* 52 */ array(1, 22, 24, 30, 35, 36, 37, 38, 39, 40, 41, 42, 43, 44, 45, 46, 47, 48, 60, 61, 62, 63, 64, 65, 66, 67, 68, 69, 70, 71, ),
        /* 53 */ array(1, 4, 24, 30, 35, 36, 37, 38, 39, 40, 41, 42, 43, 44, 45, 46, 47, 48, 60, 61, 62, 63, 64, 65, 66, 67, 68, 69, 70, 71, ),
        /* 54 */ array(1, 24, 30, 35, 36, 37, 38, 39, 40, 41, 42, 43, 44, 45, 46, 47, 48, 60, 61, 62, 63, 64, 65, 66, 67, 68, 69, 70, 71, ),
        /* 55 */ array(1, 24, 30, 35, 36, 37, 38, 39, 40, 41, 42, 43, 44, 45, 46, 47, 48, 60, 61, 62, 63, 64, 65, 66, 67, 68, 69, 70, 71, ),
        /* 56 */ array(1, 24, 30, 35, 36, 37, 38, 39, 40, 41, 42, 43, 44, 45, 46, 47, 48, 60, 61, 62, 63, 64, 65, 66, 67, 68, 69, 70, 71, ),
        /* 57 */ array(1, 24, 30, 35, 36, 37, 38, 39, 40, 41, 42, 43, 44, 45, 46, 47, 48, 60, 61, 62, 63, 64, 65, 66, 67, 68, 69, 70, 71, ),
        /* 58 */ array(1, 19, 35, 36, 37, 38, 39, 40, 41, 42, 43, 44, 45, 46, 47, 48, 60, 61, 62, 63, 64, 65, 66, 67, 68, 69, 70, 71, ),
        /* 59 */ array(1, 4, 35, 36, 37, 38, 39, 40, 41, 42, 43, 44, 45, 46, 47, 48, 60, 61, 62, 63, 64, 65, 66, 67, 68, 69, 70, 71, ),
        /* 60 */ array(1, 34, 35, 36, 37, 38, 39, 40, 41, 42, 43, 44, 45, 46, 47, 48, 60, 61, 62, 63, 64, 65, 66, 67, 68, 69, 70, 71, ),
        /* 61 */ array(1, 2, 35, 36, 37, 38, 39, 40, 41, 42, 43, 44, 45, 46, 47, 48, 60, 61, 62, 63, 64, 65, 66, 67, 68, 69, 70, 71, ),
        /* 62 */ array(1, 4, 35, 36, 37, 38, 39, 40, 41, 42, 43, 44, 45, 46, 47, 48, 60, 61, 62, 63, 64, 65, 66, 67, 68, 69, 70, 71, ),
        /* 63 */ array(1, 25, 35, 36, 37, 38, 39, 40, 41, 42, 43, 44, 45, 46, 47, 48, 60, 61, 62, 63, 64, 65, 66, 67, 68, 69, 70, 71, ),
        /* 64 */ array(1, 34, 35, 36, 37, 38, 39, 40, 41, 42, 43, 44, 45, 46, 47, 48, 60, 61, 62, 63, 64, 65, 66, 67, 68, 69, 70, 71, ),
        /* 65 */ array(1, 4, 35, 36, 37, 38, 39, 40, 41, 42, 43, 44, 45, 46, 47, 48, 60, 61, 62, 63, 64, 65, 66, 67, 68, 69, 70, 71, ),
        /* 66 */ array(1, 35, 36, 37, 38, 39, 40, 41, 42, 43, 44, 45, 46, 47, 48, 58, 60, 61, 62, 63, 64, 65, 66, 67, 68, 69, 70, 71, ),
        /* 67 */ array(1, 4, 35, 36, 37, 38, 39, 40, 41, 42, 43, 44, 45, 46, 47, 48, 60, 61, 62, 63, 64, 65, 66, 67, 68, 69, 70, 71, ),
        /* 68 */ array(1, 35, 36, 37, 38, 39, 40, 41, 42, 43, 44, 45, 46, 47, 48, 60, 61, 62, 63, 64, 65, 66, 67, 68, 69, 70, 71, ),
        /* 69 */ array(1, 35, 36, 37, 38, 39, 40, 41, 42, 43, 44, 45, 46, 47, 48, 60, 61, 62, 63, 64, 65, 66, 67, 68, 69, 70, 71, ),
        /* 70 */ array(1, 35, 36, 37, 38, 39, 40, 41, 42, 43, 44, 45, 46, 47, 48, 60, 61, 62, 63, 64, 65, 66, 67, 68, 69, 70, 71, ),
        /* 71 */ array(1, 35, 36, 37, 38, 39, 40, 41, 42, 43, 44, 45, 46, 47, 48, 60, 61, 62, 63, 64, 65, 66, 67, 68, 69, 70, 71, ),
        /* 72 */ array(1, 35, 36, 37, 38, 39, 40, 41, 42, 43, 44, 45, 46, 47, 48, 60, 61, 62, 63, 64, 65, 66, 67, 68, 69, 70, 71, ),
        /* 73 */ array(1, 35, 36, 37, 38, 39, 40, 41, 42, 43, 44, 45, 46, 47, 48, 60, 61, 62, 63, 64, 65, 66, 67, 68, 69, 70, 71, ),
        /* 74 */ array(1, 35, 36, 37, 38, 39, 40, 41, 42, 43, 44, 45, 46, 47, 48, 60, 61, 62, 63, 64, 65, 66, 67, 68, 69, 70, 71, ),
        /* 75 */ array(1, 35, 36, 37, 38, 39, 40, 41, 42, 43, 44, 45, 46, 47, 48, 60, 61, 62, 63, 64, 65, 66, 67, 68, 69, 70, 71, ),
        /* 76 */ array(1, 35, 36, 37, 38, 39, 40, 41, 42, 43, 44, 45, 46, 47, 48, 60, 61, 62, 63, 64, 65, 66, 67, 68, 69, 70, 71, ),
        /* 77 */ array(1, 35, 36, 37, 38, 39, 40, 41, 42, 43, 44, 45, 46, 47, 48, 60, 61, 62, 63, 64, 65, 66, 67, 68, 69, 70, 71, ),
        /* 78 */ array(1, 35, 36, 37, 38, 39, 40, 41, 42, 43, 44, 45, 46, 47, 48, 60, 61, 62, 63, 64, 65, 66, 67, 68, 69, 70, 71, ),
        /* 79 */ array(1, 35, 36, 37, 38, 39, 40, 41, 42, 43, 44, 45, 46, 47, 48, 60, 61, 62, 63, 64, 65, 66, 67, 68, 69, 70, 71, ),
        /* 80 */ array(1, 35, 36, 37, 38, 39, 40, 41, 42, 43, 44, 45, 46, 47, 48, 60, 61, 62, 63, 64, 65, 66, 67, 68, 69, 70, 71, ),
        /* 81 */ array(1, 35, 36, 37, 38, 39, 40, 41, 42, 43, 44, 45, 46, 47, 48, 60, 61, 62, 63, 64, 65, 66, 67, 68, 69, 70, 71, ),
        /* 82 */ array(1, 35, 36, 37, 38, 39, 40, 41, 42, 43, 44, 45, 46, 47, 48, 60, 61, 62, 63, 64, 65, 66, 67, 68, 69, 70, 71, ),
        /* 83 */ array(1, 35, 36, 37, 38, 39, 40, 41, 42, 43, 44, 45, 46, 47, 48, 60, 61, 62, 63, 64, 65, 66, 67, 68, 69, 70, 71, ),
        /* 84 */ array(1, 35, 36, 37, 38, 39, 40, 41, 42, 43, 44, 45, 46, 47, 48, 60, 61, 62, 63, 64, 65, 66, 67, 68, 69, 70, 71, ),
        /* 85 */ array(1, 35, 36, 37, 38, 39, 40, 41, 42, 43, 44, 45, 46, 47, 48, 60, 61, 62, 63, 64, 65, 66, 67, 68, 69, 70, 71, ),
        /* 86 */ array(1, 35, 36, 37, 38, 39, 40, 41, 42, 43, 44, 45, 46, 47, 48, 60, 61, 62, 63, 64, 65, 66, 67, 68, 69, 70, 71, ),
        /* 87 */ array(1, 35, 36, 37, 38, 39, 40, 41, 42, 43, 44, 45, 46, 47, 48, 60, 61, 62, 63, 64, 65, 66, 67, 68, 69, 70, 71, ),
        /* 88 */ array(35, 36, 37, 38, 39, 40, 41, 42, 43, 44, 45, 46, 47, 48, 60, 61, 62, 63, 64, 65, 66, 67, 68, 69, 70, 71, ),
        /* 89 */ array(35, 36, 37, 38, 39, 40, 41, 42, 43, 44, 45, 46, 47, 48, 60, 61, 62, 63, 64, 65, 66, 67, 68, 69, 70, 71, ),
        /* 90 */ array(5, 6, 7, 8, 9, 10, 13, 17, 18, 23, 27, 29, ),
        /* 91 */ array(1, 4, 24, 30, 33, ),
        /* 92 */ array(1, 4, 24, 30, 49, ),
        /* 93 */ array(5, 6, 7, 8, 9, 10, 13, 17, 18, 23, 27, 29, ),
        /* 94 */ array(6, 13, 17, 18, 23, 27, 29, 72, 73, ),
        /* 95 */ array(13, 16, 24, 26, 30, ),
        /* 96 */ array(13, 16, 24, 26, 30, ),
        /* 97 */ array(1, 4, 24, 30, ),
        /* 98 */ array(13, 16, 24, 30, ),
        /* 99 */ array(13, 16, 24, 30, ),
        /* 100 */ array(1, 4, 24, 30, ),
        /* 101 */ array(1, 24, 30, ),
        /* 102 */ array(1, 2, ),
        /* 103 */ array(15, 33, ),
        /* 104 */ array(6, 13, 17, 18, 23, 27, 29, 72, 73, ),
        /* 105 */ array(1, 4, 24, 25, 30, 49, ),
        /* 106 */ array(1, 4, 24, 30, 49, ),
        /* 107 */ array(1, 4, 24, 30, 49, ),
        /* 108 */ array(53, 56, 57, 59, ),
        /* 109 */ array(13, 14, 16, 20, ),
        /* 110 */ array(10, 11, 12, ),
        /* 111 */ array(53, 57, 59, ),
        /* 112 */ array(1, 4, 49, ),
        /* 113 */ array(15, 16, 56, ),
        /* 114 */ array(1, 26, 49, ),
        /* 115 */ array(13, 16, ),
        /* 116 */ array(13, 16, ),
        /* 117 */ array(13, 16, ),
        /* 118 */ array(13, 16, ),
        /* 119 */ array(13, 16, ),
        /* 120 */ array(13, 16, ),
        /* 121 */ array(15, 33, ),
        /* 122 */ array(1, 49, ),
        /* 123 */ array(15, 33, ),
        /* 124 */ array(24, 30, ),
        /* 125 */ array(13, 16, ),
        /* 126 */ array(13, 16, ),
        /* 127 */ array(24, 30, ),
        /* 128 */ array(13, 16, ),
        /* 129 */ array(13, 16, ),
        /* 130 */ array(24, 30, ),
        /* 131 */ array(13, 16, ),
        /* 132 */ array(13, 16, ),
        /* 133 */ array(13, 16, ),
        /* 134 */ array(1, ),
        /* 135 */ array(1, ),
        /* 136 */ array(1, ),
        /* 137 */ array(1, ),
        /* 138 */ array(1, ),
        /* 139 */ array(1, ),
        /* 140 */ array(1, ),
        /* 141 */ array(1, ),
        /* 142 */ array(1, ),
        /* 143 */ array(),
        /* 144 */ array(13, 14, 16, 55, ),
        /* 145 */ array(14, 53, 57, ),
        /* 146 */ array(13, 14, 16, ),
        /* 147 */ array(13, 16, 55, ),
        /* 148 */ array(13, 16, 55, ),
        /* 149 */ array(13, 15, 16, ),
        /* 150 */ array(24, 30, ),
        /* 151 */ array(53, 57, ),
        /* 152 */ array(24, 30, ),
        /* 153 */ array(24, 30, ),
        /* 154 */ array(24, 30, ),
        /* 155 */ array(24, 30, ),
        /* 156 */ array(24, 30, ),
        /* 157 */ array(24, 30, ),
        /* 158 */ array(24, 30, ),
        /* 159 */ array(53, 57, ),
        /* 160 */ array(24, 30, ),
        /* 161 */ array(24, 30, ),
        /* 162 */ array(24, 30, ),
        /* 163 */ array(24, 30, ),
        /* 164 */ array(24, 30, ),
        /* 165 */ array(24, 30, ),
        /* 166 */ array(13, 16, ),
        /* 167 */ array(24, 30, ),
        /* 168 */ array(24, 30, ),
        /* 169 */ array(24, 30, ),
        /* 170 */ array(1, ),
        /* 171 */ array(1, ),
        /* 172 */ array(24, ),
        /* 173 */ array(1, ),
        /* 174 */ array(11, ),
        /* 175 */ array(24, ),
        /* 176 */ array(2, ),
        /* 177 */ array(2, ),
        /* 178 */ array(1, ),
        /* 179 */ array(),
        /* 180 */ array(),
        /* 181 */ array(),
        /* 182 */ array(4, 19, 21, 22, 24, 25, 30, 32, 33, 34, 49, 58, ),
        /* 183 */ array(4, 14, 24, 30, 33, ),
        /* 184 */ array(13, 15, 16, 31, ),
        /* 185 */ array(4, 14, 20, ),
        /* 186 */ array(33, 53, 58, ),
        /* 187 */ array(26, 33, ),
        /* 188 */ array(13, 33, ),
        /* 189 */ array(19, 32, ),
        /* 190 */ array(2, 14, ),
        /* 191 */ array(32, 34, ),
        /* 192 */ array(14, 53, ),
        /* 193 */ array(32, 58, ),
        /* 194 */ array(14, 20, ),
        /* 195 */ array(16, 55, ),
        /* 196 */ array(32, 34, ),
        /* 197 */ array(32, 34, ),
        /* 198 */ array(34, ),
        /* 199 */ array(4, ),
        /* 200 */ array(58, ),
        /* 201 */ array(16, ),
        /* 202 */ array(4, ),
        /* 203 */ array(16, ),
        /* 204 */ array(16, ),
        /* 205 */ array(56, ),
        /* 206 */ array(16, ),
        /* 207 */ array(33, ),
        /* 208 */ array(14, ),
        /* 209 */ array(16, ),
        /* 210 */ array(4, ),
        /* 211 */ array(2, ),
        /* 212 */ array(31, ),
        /* 213 */ array(20, ),
        /* 214 */ array(21, ),
        /* 215 */ array(15, ),
        /* 216 */ array(16, ),
        /* 217 */ array(33, ),
        /* 218 */ array(53, ),
        /* 219 */ array(16, ),
        /* 220 */ array(31, ),
        /* 221 */ array(15, ),
        /* 222 */ array(15, ),
        /* 223 */ array(33, ),
        /* 224 */ array(15, ),
        /* 225 */ array(50, ),
        /* 226 */ array(15, ),
        /* 227 */ array(16, ),
        /* 228 */ array(2, ),
        /* 229 */ array(),
        /* 230 */ array(),
        /* 231 */ array(),
        /* 232 */ array(),
        /* 233 */ array(),
        /* 234 */ array(),
        /* 235 */ array(),
        /* 236 */ array(),
        /* 237 */ array(),
        /* 238 */ array(),
        /* 239 */ array(),
        /* 240 */ array(),
        /* 241 */ array(),
        /* 242 */ array(),
        /* 243 */ array(),
        /* 244 */ array(),
        /* 245 */ array(),
        /* 246 */ array(),
        /* 247 */ array(),
        /* 248 */ array(),
        /* 249 */ array(),
        /* 250 */ array(),
        /* 251 */ array(),
        /* 252 */ array(),
        /* 253 */ array(),
        /* 254 */ array(),
        /* 255 */ array(),
        /* 256 */ array(),
        /* 257 */ array(),
        /* 258 */ array(),
        /* 259 */ array(),
        /* 260 */ array(),
        /* 261 */ array(),
        /* 262 */ array(),
        /* 263 */ array(),
        /* 264 */ array(),
        /* 265 */ array(),
        /* 266 */ array(),
        /* 267 */ array(),
        /* 268 */ array(),
        /* 269 */ array(),
        /* 270 */ array(),
        /* 271 */ array(),
        /* 272 */ array(),
        /* 273 */ array(),
        /* 274 */ array(),
        /* 275 */ array(),
        /* 276 */ array(),
        /* 277 */ array(),
        /* 278 */ array(),
        /* 279 */ array(),
        /* 280 */ array(),
        /* 281 */ array(),
        /* 282 */ array(),
        /* 283 */ array(),
        /* 284 */ array(),
        /* 285 */ array(),
        /* 286 */ array(),
        /* 287 */ array(),
        /* 288 */ array(),
        /* 289 */ array(),
        /* 290 */ array(),
        /* 291 */ array(),
        /* 292 */ array(),
        /* 293 */ array(),
        /* 294 */ array(),
        /* 295 */ array(),
        /* 296 */ array(),
        /* 297 */ array(),
        /* 298 */ array(),
        /* 299 */ array(),
        /* 300 */ array(),
        /* 301 */ array(),
        /* 302 */ array(),
        /* 303 */ array(),
        /* 304 */ array(),
        /* 305 */ array(),
        /* 306 */ array(),
        /* 307 */ array(),
        /* 308 */ array(),
        /* 309 */ array(),
        /* 310 */ array(),
        /* 311 */ array(),
        /* 312 */ array(),
        /* 313 */ array(),
        /* 314 */ array(),
        /* 315 */ array(),
        /* 316 */ array(),
        /* 317 */ array(),
        /* 318 */ array(),
        /* 319 */ array(),
);
    static public $yy_default = array(
 /*     0 */   322,  484,  499,  460,  499,  460,  499,  460,  499,  499,
 /*    10 */   499,  499,  499,  499,  499,  499,  499,  499,  499,  499,
 /*    20 */   499,  499,  499,  499,  499,  499,  499,  499,  499,  499,
 /*    30 */   499,  499,  499,  499,  499,  499,  499,  499,  499,  499,
 /*    40 */   499,  499,  499,  499,  499,  499,  499,  499,  499,  499,
 /*    50 */   499,  499,  372,  499,  372,  346,  372,  351,  499,  499,
 /*    60 */   499,  499,  499,  377,  499,  499,  499,  499,  397,  398,
 /*    70 */   406,  405,  401,  402,  459,  486,  487,  379,  396,  383,
 /*    80 */   458,  485,  382,  344,  393,  377,  356,  374,  409,  410,
 /*    90 */   325,  420,  385,  320,  499,  372,  372,  392,  372,  372,
 /*   100 */   426,  372,  365,  499,  499,  385,  385,  385,  499,  499,
 /*   110 */   499,  427,  385,  499,  385,  499,  499,  499,  499,  499,
 /*   120 */   499,  499,  385,  499,  353,  499,  499,  372,  499,  499,
 /*   130 */   372,  499,  499,  499,  413,  412,  408,  395,  390,  414,
 /*   140 */   388,  389,  368,  467,  428,  432,  499,  438,  428,  499,
 /*   150 */   363,  431,  343,  364,  361,  359,  342,  362,  348,  432,
 /*   160 */   358,  352,  340,  354,  360,  350,  451,  355,  347,  341,
 /*   170 */   369,  366,  498,  392,  336,  498,  461,  462,  426,  466,
 /*   180 */   466,  466,  387,  420,  499,  411,  420,  420,  450,  499,
 /*   190 */   448,  499,  416,  499,  411,  499,  499,  499,  499,  499,
 /*   200 */   499,  499,  499,  499,  499,  499,  499,  448,  499,  499,
 /*   210 */   499,  448,  418,  411,  380,  499,  499,  407,  416,  499,
 /*   220 */   499,  499,  499,  420,  499,  422,  499,  499,  448,  454,
 /*   230 */   323,  321,  452,  468,  449,  497,  442,  373,  444,  437,
 /*   240 */   422,  384,  443,  381,  435,  445,  455,  357,  404,  419,
 /*   250 */   421,  386,  417,  415,  463,  464,  387,  434,  429,  433,
 /*   260 */   430,  425,  424,  446,  423,  465,  448,  329,  330,  331,
 /*   270 */   328,  327,  324,  326,  332,  333,  370,  371,  447,  338,
 /*   280 */   337,  334,  335,  439,  440,  488,  489,  490,  367,  493,
 /*   290 */   391,  494,  492,  495,  482,  453,  456,  483,  481,  496,
 /*   300 */   491,  480,  479,  400,  403,  469,  399,  436,  441,  394,
 /*   310 */   470,  471,  476,  477,  478,  475,  474,  472,  473,  457,
);
    const YYNOCODE = 115;
    const YYSTACKDEPTH = 500;
    const YYNSTATE = 320;
    const YYNRULE = 179;
    const YYERRORSYMBOL = 74;
    const YYERRSYMDT = 'yy0';
    const YYFALLBACK = 0;
    public static $yyFallback = array(
    );

    public $yyTraceFILE;
    public $yyTracePrompt;
    public $yyidx;                    /* Index of top element in stack */
    public $yyerrcnt;                 /* Shifts left before out of the error */
    public $yystack = array();  /* The parser's stack */

    public $yyTokenName = array(
  '$',             'VERT',          'COLON',         'SETSTRICT',   
  'RDEL',          'COMMENT',       'TEXT',          'STRIPON',     
  'STRIPOFF',      'BLOCKSOURCE',   'LITERALSTART',  'LITERALEND',  
  'LITERAL',       'LDEL',          'EQUAL',         'DOLLAR',      
  'ID',            'LDELIF',        'LDELFOR',       'SEMICOLON',   
  'INCDEC',        'TO',            'STEP',          'LDELFOREACH', 
  'SPACE',         'AS',            'APTR',          'LDELSETFILTER',
  'SMARTYBLOCKCHILDPARENT',  'LDELSLASH',     'ATTR',          'INTEGER',     
  'COMMA',         'OPENP',         'CLOSEP',        'MATH',        
  'UNIMATH',       'ANDSYM',        'ISIN',          'ISDIVBY',     
  'ISNOTDIVBY',    'ISEVEN',        'ISNOTEVEN',     'ISEVENBY',    
  'ISNOTEVENBY',   'ISODD',         'ISNOTODD',      'ISODDBY',     
  'ISNOTODDBY',    'INSTANCEOF',    'QMARK',         'NOT',         
  'TYPECAST',      'DOT',           'SINGLEQUOTESTRING',  'AT',          
  'HATCH',         'OPENB',         'CLOSEB',        'PTR',         
  'EQUALS',        'NOTEQUALS',     'GREATERTHAN',   'LESSTHAN',    
  'GREATEREQUAL',  'LESSEQUAL',     'IDENTITY',      'NONEIDENTITY',
  'MOD',           'LAND',          'LOR',           'LXOR',        
  'QUOTE',         'DOLLARID',      'error',         'start',       
  'strictmode',    'template',      'template_element',  'smartytag',   
  'literal',       'literal_elements',  'literal_element',  'value',       
  'modifierlist',  'attributes',    'expr',          'variable',    
  'statement',     'statements',    'optspace',      'varvar',      
  'foraction',     'modparameters',  'attribute',     'variablebase',
  'ternary',       'array',         'ifcond',        'lop',         
  'function',      'doublequoted_with_quotes',  'variableinternal',  'indexdef',    
  'objectelement',  'varvarele',     'method',        'params',      
  'modifier',      'modparameter',  'arrayelements',  'arrayelement',
  'doublequoted',  'doublequotedcontent',
    );

    public static $yyRuleName = array(
 /*   0 */ "start ::= strictmode template",
 /*   1 */ "strictmode ::= SETSTRICT",
 /*   2 */ "strictmode ::=",
 /*   3 */ "template ::= template_element",
 /*   4 */ "template ::= template template_element",
 /*   5 */ "template ::=",
 /*   6 */ "template_element ::= smartytag RDEL",
 /*   7 */ "template_element ::= COMMENT",
 /*   8 */ "template_element ::= literal",
 /*   9 */ "template_element ::= TEXT",
 /*  10 */ "template_element ::= STRIPON",
 /*  11 */ "template_element ::= STRIPOFF",
 /*  12 */ "template_element ::= BLOCKSOURCE",
 /*  13 */ "literal ::= LITERALSTART LITERALEND",
 /*  14 */ "literal ::= LITERALSTART literal_elements LITERALEND",
 /*  15 */ "literal_elements ::= literal_elements literal_element",
 /*  16 */ "literal_elements ::=",
 /*  17 */ "literal_element ::= literal",
 /*  18 */ "literal_element ::= LITERAL",
 /*  19 */ "smartytag ::= LDEL value",
 /*  20 */ "smartytag ::= LDEL value modifierlist attributes",
 /*  21 */ "smartytag ::= LDEL value attributes",
 /*  22 */ "smartytag ::= LDEL expr modifierlist attributes",
 /*  23 */ "smartytag ::= LDEL expr attributes",
 /*  24 */ "smartytag ::= LDEL variable EQUAL expr",
 /*  25 */ "smartytag ::= LDEL DOLLAR ID EQUAL value",
 /*  26 */ "smartytag ::= LDEL DOLLAR ID EQUAL expr",
 /*  27 */ "smartytag ::= LDEL DOLLAR ID EQUAL expr attributes",
 /*  28 */ "smartytag ::= LDEL ID attributes",
 /*  29 */ "smartytag ::= LDEL ID",
 /*  30 */ "smartytag ::= LDEL ID modifierlist attributes",
 /*  31 */ "smartytag ::= LDELIF expr",
 /*  32 */ "smartytag ::= LDELIF expr attributes",
 /*  33 */ "smartytag ::= LDELIF statement",
 /*  34 */ "smartytag ::= LDELIF statement attributes",
 /*  35 */ "smartytag ::= LDELFOR statements SEMICOLON optspace expr SEMICOLON optspace DOLLAR varvar foraction attributes",
 /*  36 */ "foraction ::= EQUAL expr",
 /*  37 */ "foraction ::= INCDEC",
 /*  38 */ "smartytag ::= LDELFOR statement TO expr attributes",
 /*  39 */ "smartytag ::= LDELFOR statement TO expr STEP expr attributes",
 /*  40 */ "smartytag ::= LDELFOREACH attributes",
 /*  41 */ "smartytag ::= LDELFOREACH SPACE value AS DOLLAR varvar attributes",
 /*  42 */ "smartytag ::= LDELFOREACH SPACE value AS DOLLAR varvar APTR DOLLAR varvar attributes",
 /*  43 */ "smartytag ::= LDELFOREACH SPACE expr AS DOLLAR varvar attributes",
 /*  44 */ "smartytag ::= LDELFOREACH SPACE expr AS DOLLAR varvar APTR DOLLAR varvar attributes",
 /*  45 */ "smartytag ::= LDELSETFILTER ID modparameters",
 /*  46 */ "smartytag ::= LDELSETFILTER ID modparameters modifierlist",
 /*  47 */ "smartytag ::= LDEL SMARTYBLOCKCHILDPARENT",
 /*  48 */ "smartytag ::= LDELSLASH ID",
 /*  49 */ "smartytag ::= LDELSLASH ID modifierlist",
 /*  50 */ "attributes ::= attributes attribute",
 /*  51 */ "attributes ::= attribute",
 /*  52 */ "attributes ::=",
 /*  53 */ "attribute ::= SPACE ID EQUAL ID",
 /*  54 */ "attribute ::= ATTR expr",
 /*  55 */ "attribute ::= ATTR value",
 /*  56 */ "attribute ::= SPACE ID",
 /*  57 */ "attribute ::= SPACE expr",
 /*  58 */ "attribute ::= SPACE value",
 /*  59 */ "attribute ::= SPACE INTEGER EQUAL expr",
 /*  60 */ "statements ::= statement",
 /*  61 */ "statements ::= statements COMMA statement",
 /*  62 */ "statement ::= DOLLAR varvar EQUAL expr",
 /*  63 */ "statement ::= variablebase EQUAL expr",
 /*  64 */ "statement ::= OPENP statement CLOSEP",
 /*  65 */ "expr ::= value",
 /*  66 */ "expr ::= ternary",
 /*  67 */ "expr ::= DOLLAR ID COLON ID",
 /*  68 */ "expr ::= expr MATH value",
 /*  69 */ "expr ::= expr UNIMATH value",
 /*  70 */ "expr ::= expr ANDSYM value",
 /*  71 */ "expr ::= array",
 /*  72 */ "expr ::= expr modifierlist",
 /*  73 */ "expr ::= expr ifcond expr",
 /*  74 */ "expr ::= expr ISIN array",
 /*  75 */ "expr ::= expr ISIN value",
 /*  76 */ "expr ::= expr lop expr",
 /*  77 */ "expr ::= expr ISDIVBY expr",
 /*  78 */ "expr ::= expr ISNOTDIVBY expr",
 /*  79 */ "expr ::= expr ISEVEN",
 /*  80 */ "expr ::= expr ISNOTEVEN",
 /*  81 */ "expr ::= expr ISEVENBY expr",
 /*  82 */ "expr ::= expr ISNOTEVENBY expr",
 /*  83 */ "expr ::= expr ISODD",
 /*  84 */ "expr ::= expr ISNOTODD",
 /*  85 */ "expr ::= expr ISODDBY expr",
 /*  86 */ "expr ::= expr ISNOTODDBY expr",
 /*  87 */ "expr ::= value INSTANCEOF ID",
 /*  88 */ "expr ::= value INSTANCEOF value",
 /*  89 */ "ternary ::= OPENP expr CLOSEP QMARK DOLLAR ID COLON expr",
 /*  90 */ "ternary ::= OPENP expr CLOSEP QMARK expr COLON expr",
 /*  91 */ "value ::= variable",
 /*  92 */ "value ::= UNIMATH value",
 /*  93 */ "value ::= NOT value",
 /*  94 */ "value ::= TYPECAST value",
 /*  95 */ "value ::= variable INCDEC",
 /*  96 */ "value ::= INTEGER",
 /*  97 */ "value ::= INTEGER DOT INTEGER",
 /*  98 */ "value ::= INTEGER DOT",
 /*  99 */ "value ::= DOT INTEGER",
 /* 100 */ "value ::= ID",
 /* 101 */ "value ::= function",
 /* 102 */ "value ::= OPENP expr CLOSEP",
 /* 103 */ "value ::= SINGLEQUOTESTRING",
 /* 104 */ "value ::= doublequoted_with_quotes",
 /* 105 */ "value ::= smartytag RDEL",
 /* 106 */ "value ::= value modifierlist",
 /* 107 */ "variable ::= variableinternal",
 /* 108 */ "variablebase ::= DOLLAR varvar",
 /* 109 */ "variableinternal ::= variableinternal indexdef",
 /* 110 */ "variableinternal ::= variablebase indexdef indexdef",
 /* 111 */ "variableinternal ::= variablebase indexdef",
 /* 112 */ "variableinternal ::= variablebase",
 /* 113 */ "variableinternal ::= variableinternal objectelement",
 /* 114 */ "variableinternal ::= DOLLAR varvar AT ID",
 /* 115 */ "variableinternal ::= HATCH ID HATCH",
 /* 116 */ "variableinternal ::= HATCH variableinternal HATCH",
 /* 117 */ "indexdef ::= OPENB CLOSEB",
 /* 118 */ "indexdef ::= DOT DOLLAR varvar",
 /* 119 */ "indexdef ::= DOT DOLLAR varvar AT ID",
 /* 120 */ "indexdef ::= DOT ID",
 /* 121 */ "indexdef ::= DOT INTEGER",
 /* 122 */ "indexdef ::= DOT LDEL expr RDEL",
 /* 123 */ "indexdef ::= OPENB ID CLOSEB",
 /* 124 */ "indexdef ::= OPENB ID DOT ID CLOSEB",
 /* 125 */ "indexdef ::= OPENB expr CLOSEB",
 /* 126 */ "varvar ::= varvarele",
 /* 127 */ "varvar ::= varvar varvarele",
 /* 128 */ "varvarele ::= ID",
 /* 129 */ "varvarele ::= LDEL expr RDEL",
 /* 130 */ "objectelement ::= PTR ID",
 /* 131 */ "objectelement ::= PTR DOLLAR varvar",
 /* 132 */ "objectelement ::= PTR LDEL expr RDEL",
 /* 133 */ "objectelement ::= PTR ID LDEL expr RDEL",
 /* 134 */ "objectelement ::= PTR method",
 /* 135 */ "function ::= ID OPENP params CLOSEP",
 /* 136 */ "method ::= ID OPENP params CLOSEP",
 /* 137 */ "method ::= DOLLAR ID OPENP params CLOSEP",
 /* 138 */ "params ::= params COMMA expr",
 /* 139 */ "params ::= expr",
 /* 140 */ "params ::=",
 /* 141 */ "modifierlist ::= modifierlist modifier modparameters",
 /* 142 */ "modifierlist ::= modifier modparameters",
 /* 143 */ "modifier ::= VERT AT ID",
 /* 144 */ "modifier ::= VERT ID",
 /* 145 */ "modparameters ::= modparameters modparameter",
 /* 146 */ "modparameters ::=",
 /* 147 */ "modparameter ::= COLON value",
 /* 148 */ "modparameter ::= COLON array",
 /* 149 */ "ifcond ::= EQUALS",
 /* 150 */ "ifcond ::= NOTEQUALS",
 /* 151 */ "ifcond ::= GREATERTHAN",
 /* 152 */ "ifcond ::= LESSTHAN",
 /* 153 */ "ifcond ::= GREATEREQUAL",
 /* 154 */ "ifcond ::= LESSEQUAL",
 /* 155 */ "ifcond ::= IDENTITY",
 /* 156 */ "ifcond ::= NONEIDENTITY",
 /* 157 */ "ifcond ::= MOD",
 /* 158 */ "lop ::= LAND",
 /* 159 */ "lop ::= LOR",
 /* 160 */ "lop ::= LXOR",
 /* 161 */ "array ::= OPENB arrayelements CLOSEB",
 /* 162 */ "arrayelements ::= arrayelement",
 /* 163 */ "arrayelements ::= arrayelements COMMA arrayelement",
 /* 164 */ "arrayelements ::=",
 /* 165 */ "arrayelement ::= value APTR expr",
 /* 166 */ "arrayelement ::= ID APTR expr",
 /* 167 */ "arrayelement ::= expr",
 /* 168 */ "doublequoted_with_quotes ::= QUOTE QUOTE",
 /* 169 */ "doublequoted_with_quotes ::= QUOTE doublequoted QUOTE",
 /* 170 */ "doublequoted ::= doublequoted doublequotedcontent",
 /* 171 */ "doublequoted ::= doublequotedcontent",
 /* 172 */ "doublequotedcontent ::= DOLLARID",
 /* 173 */ "doublequotedcontent ::= LDEL variable RDEL",
 /* 174 */ "doublequotedcontent ::= LDEL expr RDEL",
 /* 175 */ "doublequotedcontent ::= smartytag RDEL",
 /* 176 */ "doublequotedcontent ::= TEXT",
 /* 177 */ "optspace ::= SPACE",
 /* 178 */ "optspace ::=",
    );

    public function tokenName($tokenType)
    {
        if ($tokenType === 0) {
            return 'End of Input';
        }
        if ($tokenType > 0 && $tokenType < count($this->yyTokenName)) {
            return $this->yyTokenName[$tokenType];
        } else {
            return "Unknown";
        }
    }

    public static function yy_destructor($yymajor, $yypminor)
    {
        switch ($yymajor) {
            default:  break;   /* If no destructor action specified: do nothing */
        }
    }

    public function yy_pop_parser_stack()
    {
        if (!count($this->yystack)) {
            return;
        }
        $yytos = array_pop($this->yystack);
        if ($this->yyTraceFILE && $this->yyidx >= 0) {
            fwrite($this->yyTraceFILE,
                $this->yyTracePrompt . 'Popping ' . $this->yyTokenName[$yytos->major] .
                    "\n");
        }
        $yymajor = $yytos->major;
        self::yy_destructor($yymajor, $yytos->minor);
        $this->yyidx--;

        return $yymajor;
    }

    public function __destruct()
    {
        while ($this->yystack !== Array()) {
            $this->yy_pop_parser_stack();
        }
        if (is_resource($this->yyTraceFILE)) {
            fclose($this->yyTraceFILE);
        }
    }

    public function yy_get_expected_tokens($token)
    {
        $state = $this->yystack[$this->yyidx]->stateno;
        $expected = self::$yyExpectedTokens[$state];
        if (in_array($token, self::$yyExpectedTokens[$state], true)) {
            return $expected;
        }
        $stack = $this->yystack;
        $yyidx = $this->yyidx;
        do {
            $yyact = $this->yy_find_shift_action($token);
            if ($yyact >= self::YYNSTATE && $yyact < self::YYNSTATE + self::YYNRULE) {
                // reduce action
                $done = 0;
                do {
                    if ($done++ == 100) {
                        $this->yyidx = $yyidx;
                        $this->yystack = $stack;
                        // too much recursion prevents proper detection
                        // so give up
                        return array_unique($expected);
                    }
                    $yyruleno = $yyact - self::YYNSTATE;
                    $this->yyidx -= self::$yyRuleInfo[$yyruleno]['rhs'];
                    $nextstate = $this->yy_find_reduce_action(
                        $this->yystack[$this->yyidx]->stateno,
                        self::$yyRuleInfo[$yyruleno]['lhs']);
                    if (isset(self::$yyExpectedTokens[$nextstate])) {
                $expected = array_merge($expected, self::$yyExpectedTokens[$nextstate]);
                            if (in_array($token,
                                  self::$yyExpectedTokens[$nextstate], true)) {
                            $this->yyidx = $yyidx;
                            $this->yystack = $stack;

                            return array_unique($expected);
                        }
                    }
                    if ($nextstate < self::YYNSTATE) {
                        // we need to shift a non-terminal
                        $this->yyidx++;
                        $x = new TP_yyStackEntry;
                        $x->stateno = $nextstate;
                        $x->major = self::$yyRuleInfo[$yyruleno]['lhs'];
                        $this->yystack[$this->yyidx] = $x;
                        continue 2;
                    } elseif ($nextstate == self::YYNSTATE + self::YYNRULE + 1) {
                        $this->yyidx = $yyidx;
                        $this->yystack = $stack;
                        // the last token was just ignored, we can't accept
                        // by ignoring input, this is in essence ignoring a
                        // syntax error!
                        return array_unique($expected);
                    } elseif ($nextstate === self::YY_NO_ACTION) {
                        $this->yyidx = $yyidx;
                        $this->yystack = $stack;
                        // input accepted, but not shifted (I guess)
                        return $expected;
                    } else {
                        $yyact = $nextstate;
                    }
                } while (true);
            }
            break;
        } while (true);
    $this->yyidx = $yyidx;
    $this->yystack = $stack;

        return array_unique($expected);
    }

    public function yy_is_expected_token($token)
    {
        if ($token === 0) {
            return true; // 0 is not part of this
        }
        $state = $this->yystack[$this->yyidx]->stateno;
        if (in_array($token, self::$yyExpectedTokens[$state], true)) {
            return true;
        }
        $stack = $this->yystack;
        $yyidx = $this->yyidx;
        do {
            $yyact = $this->yy_find_shift_action($token);
            if ($yyact >= self::YYNSTATE && $yyact < self::YYNSTATE + self::YYNRULE) {
                // reduce action
                $done = 0;
                do {
                    if ($done++ == 100) {
                        $this->yyidx = $yyidx;
                        $this->yystack = $stack;
                        // too much recursion prevents proper detection
                        // so give up
                        return true;
                    }
                    $yyruleno = $yyact - self::YYNSTATE;
                    $this->yyidx -= self::$yyRuleInfo[$yyruleno]['rhs'];
                    $nextstate = $this->yy_find_reduce_action(
                        $this->yystack[$this->yyidx]->stateno,
                        self::$yyRuleInfo[$yyruleno]['lhs']);
                    if (isset(self::$yyExpectedTokens[$nextstate]) &&
                          in_array($token, self::$yyExpectedTokens[$nextstate], true)) {
                        $this->yyidx = $yyidx;
                        $this->yystack = $stack;

                        return true;
                    }
                    if ($nextstate < self::YYNSTATE) {
                        // we need to shift a non-terminal
                        $this->yyidx++;
                        $x = new TP_yyStackEntry;
                        $x->stateno = $nextstate;
                        $x->major = self::$yyRuleInfo[$yyruleno]['lhs'];
                        $this->yystack[$this->yyidx] = $x;
                        continue 2;
                    } elseif ($nextstate == self::YYNSTATE + self::YYNRULE + 1) {
                        $this->yyidx = $yyidx;
                        $this->yystack = $stack;
                        if (!$token && $token !== '0') {
                            // end of input: this is valid
                            return true;
                        }
                        // the last token was just ignored, we can't accept
                        // by ignoring input, this is in essence ignoring a
                        // syntax error!
                        return false;
                    } elseif ($nextstate === self::YY_NO_ACTION) {
                        $this->yyidx = $yyidx;
                        $this->yystack = $stack;
                        // input accepted, but not shifted (I guess)
                        return true;
                    } else {
                        $yyact = $nextstate;
                    }
                } while (true);
            }
            break;
        } while (true);
        $this->yyidx = $yyidx;
        $this->yystack = $stack;

        return true;
    }

   public function yy_find_shift_action($iLookAhead)
    {
        $stateno = $this->yystack[$this->yyidx]->stateno;

        /* if ($this->yyidx < 0) return self::YY_NO_ACTION;  */
        if (!isset(self::$yy_shift_ofst[$stateno])) {
            // no shift actions
            return self::$yy_default[$stateno];
        }
        $i = self::$yy_shift_ofst[$stateno];
        if ($i === self::YY_SHIFT_USE_DFLT) {
            return self::$yy_default[$stateno];
        }
        if ($iLookAhead == self::YYNOCODE) {
            return self::YY_NO_ACTION;
        }
        $i += $iLookAhead;
        if ($i < 0 || $i >= self::YY_SZ_ACTTAB ||
              self::$yy_lookahead[$i] != $iLookAhead) {
            if (count(self::$yyFallback) && $iLookAhead < count(self::$yyFallback)
                   && ($iFallback = self::$yyFallback[$iLookAhead]) != 0) {
                if ($this->yyTraceFILE) {
                    fwrite($this->yyTraceFILE, $this->yyTracePrompt . "FALLBACK " .
                        $this->yyTokenName[$iLookAhead] . " => " .
                        $this->yyTokenName[$iFallback] . "\n");
                }

                return $this->yy_find_shift_action($iFallback);
            }

            return self::$yy_default[$stateno];
        } else {
            return self::$yy_action[$i];
        }
    }

    public function yy_find_reduce_action($stateno, $iLookAhead)
    {
        /* $stateno = $this->yystack[$this->yyidx]->stateno; */

        if (!isset(self::$yy_reduce_ofst[$stateno])) {
            return self::$yy_default[$stateno];
        }
        $i = self::$yy_reduce_ofst[$stateno];
        if ($i == self::YY_REDUCE_USE_DFLT) {
            return self::$yy_default[$stateno];
        }
        if ($iLookAhead == self::YYNOCODE) {
            return self::YY_NO_ACTION;
        }
        $i += $iLookAhead;
        if ($i < 0 || $i >= self::YY_SZ_ACTTAB ||
              self::$yy_lookahead[$i] != $iLookAhead) {
            return self::$yy_default[$stateno];
        } else {
            return self::$yy_action[$i];
        }
    }

    public function yy_shift($yyNewState, $yyMajor, $yypMinor)
    {
        $this->yyidx++;
        if ($this->yyidx >= self::YYSTACKDEPTH) {
            $this->yyidx--;
            if ($this->yyTraceFILE) {
                fprintf($this->yyTraceFILE, "%sStack Overflow!\n", $this->yyTracePrompt);
            }
            while ($this->yyidx >= 0) {
                $this->yy_pop_parser_stack();
            }
#line 126 "smarty_internal_templateparser.y"

    $this->internalError = true;
    $this->compiler->trigger_template_error("Stack overflow in template parser");
#line 1634 "smarty_internal_templateparser.php"

            return;
        }
        $yytos = new TP_yyStackEntry;
        $yytos->stateno = $yyNewState;
        $yytos->major = $yyMajor;
        $yytos->minor = $yypMinor;
        array_push($this->yystack, $yytos);
        if ($this->yyTraceFILE && $this->yyidx > 0) {
            fprintf($this->yyTraceFILE, "%sShift %d\n", $this->yyTracePrompt,
                $yyNewState);
            fprintf($this->yyTraceFILE, "%sStack:", $this->yyTracePrompt);
            for ($i = 1; $i <= $this->yyidx; $i++) {
                fprintf($this->yyTraceFILE, " %s",
                    $this->yyTokenName[$this->yystack[$i]->major]);
            }
            fwrite($this->yyTraceFILE,"\n");
        }
    }

    public static $yyRuleInfo = array(
  array( 'lhs' => 75, 'rhs' => 2 ),
  array( 'lhs' => 76, 'rhs' => 1 ),
  array( 'lhs' => 76, 'rhs' => 0 ),
  array( 'lhs' => 77, 'rhs' => 1 ),
  array( 'lhs' => 77, 'rhs' => 2 ),
  array( 'lhs' => 77, 'rhs' => 0 ),
  array( 'lhs' => 78, 'rhs' => 2 ),
  array( 'lhs' => 78, 'rhs' => 1 ),
  array( 'lhs' => 78, 'rhs' => 1 ),
  array( 'lhs' => 78, 'rhs' => 1 ),
  array( 'lhs' => 78, 'rhs' => 1 ),
  array( 'lhs' => 78, 'rhs' => 1 ),
  array( 'lhs' => 78, 'rhs' => 1 ),
  array( 'lhs' => 80, 'rhs' => 2 ),
  array( 'lhs' => 80, 'rhs' => 3 ),
  array( 'lhs' => 81, 'rhs' => 2 ),
  array( 'lhs' => 81, 'rhs' => 0 ),
  array( 'lhs' => 82, 'rhs' => 1 ),
  array( 'lhs' => 82, 'rhs' => 1 ),
  array( 'lhs' => 79, 'rhs' => 2 ),
  array( 'lhs' => 79, 'rhs' => 4 ),
  array( 'lhs' => 79, 'rhs' => 3 ),
  array( 'lhs' => 79, 'rhs' => 4 ),
  array( 'lhs' => 79, 'rhs' => 3 ),
  array( 'lhs' => 79, 'rhs' => 4 ),
  array( 'lhs' => 79, 'rhs' => 5 ),
  array( 'lhs' => 79, 'rhs' => 5 ),
  array( 'lhs' => 79, 'rhs' => 6 ),
  array( 'lhs' => 79, 'rhs' => 3 ),
  array( 'lhs' => 79, 'rhs' => 2 ),
  array( 'lhs' => 79, 'rhs' => 4 ),
  array( 'lhs' => 79, 'rhs' => 2 ),
  array( 'lhs' => 79, 'rhs' => 3 ),
  array( 'lhs' => 79, 'rhs' => 2 ),
  array( 'lhs' => 79, 'rhs' => 3 ),
  array( 'lhs' => 79, 'rhs' => 11 ),
  array( 'lhs' => 92, 'rhs' => 2 ),
  array( 'lhs' => 92, 'rhs' => 1 ),
  array( 'lhs' => 79, 'rhs' => 5 ),
  array( 'lhs' => 79, 'rhs' => 7 ),
  array( 'lhs' => 79, 'rhs' => 2 ),
  array( 'lhs' => 79, 'rhs' => 7 ),
  array( 'lhs' => 79, 'rhs' => 10 ),
  array( 'lhs' => 79, 'rhs' => 7 ),
  array( 'lhs' => 79, 'rhs' => 10 ),
  array( 'lhs' => 79, 'rhs' => 3 ),
  array( 'lhs' => 79, 'rhs' => 4 ),
  array( 'lhs' => 79, 'rhs' => 2 ),
  array( 'lhs' => 79, 'rhs' => 2 ),
  array( 'lhs' => 79, 'rhs' => 3 ),
  array( 'lhs' => 85, 'rhs' => 2 ),
  array( 'lhs' => 85, 'rhs' => 1 ),
  array( 'lhs' => 85, 'rhs' => 0 ),
  array( 'lhs' => 94, 'rhs' => 4 ),
  array( 'lhs' => 94, 'rhs' => 2 ),
  array( 'lhs' => 94, 'rhs' => 2 ),
  array( 'lhs' => 94, 'rhs' => 2 ),
  array( 'lhs' => 94, 'rhs' => 2 ),
  array( 'lhs' => 94, 'rhs' => 2 ),
  array( 'lhs' => 94, 'rhs' => 4 ),
  array( 'lhs' => 89, 'rhs' => 1 ),
  array( 'lhs' => 89, 'rhs' => 3 ),
  array( 'lhs' => 88, 'rhs' => 4 ),
  array( 'lhs' => 88, 'rhs' => 3 ),
  array( 'lhs' => 88, 'rhs' => 3 ),
  array( 'lhs' => 86, 'rhs' => 1 ),
  array( 'lhs' => 86, 'rhs' => 1 ),
  array( 'lhs' => 86, 'rhs' => 4 ),
  array( 'lhs' => 86, 'rhs' => 3 ),
  array( 'lhs' => 86, 'rhs' => 3 ),
  array( 'lhs' => 86, 'rhs' => 3 ),
  array( 'lhs' => 86, 'rhs' => 1 ),
  array( 'lhs' => 86, 'rhs' => 2 ),
  array( 'lhs' => 86, 'rhs' => 3 ),
  array( 'lhs' => 86, 'rhs' => 3 ),
  array( 'lhs' => 86, 'rhs' => 3 ),
  array( 'lhs' => 86, 'rhs' => 3 ),
  array( 'lhs' => 86, 'rhs' => 3 ),
  array( 'lhs' => 86, 'rhs' => 3 ),
  array( 'lhs' => 86, 'rhs' => 2 ),
  array( 'lhs' => 86, 'rhs' => 2 ),
  array( 'lhs' => 86, 'rhs' => 3 ),
  array( 'lhs' => 86, 'rhs' => 3 ),
  array( 'lhs' => 86, 'rhs' => 2 ),
  array( 'lhs' => 86, 'rhs' => 2 ),
  array( 'lhs' => 86, 'rhs' => 3 ),
  array( 'lhs' => 86, 'rhs' => 3 ),
  array( 'lhs' => 86, 'rhs' => 3 ),
  array( 'lhs' => 86, 'rhs' => 3 ),
  array( 'lhs' => 96, 'rhs' => 8 ),
  array( 'lhs' => 96, 'rhs' => 7 ),
  array( 'lhs' => 83, 'rhs' => 1 ),
  array( 'lhs' => 83, 'rhs' => 2 ),
  array( 'lhs' => 83, 'rhs' => 2 ),
  array( 'lhs' => 83, 'rhs' => 2 ),
  array( 'lhs' => 83, 'rhs' => 2 ),
  array( 'lhs' => 83, 'rhs' => 1 ),
  array( 'lhs' => 83, 'rhs' => 3 ),
  array( 'lhs' => 83, 'rhs' => 2 ),
  array( 'lhs' => 83, 'rhs' => 2 ),
  array( 'lhs' => 83, 'rhs' => 1 ),
  array( 'lhs' => 83, 'rhs' => 1 ),
  array( 'lhs' => 83, 'rhs' => 3 ),
  array( 'lhs' => 83, 'rhs' => 1 ),
  array( 'lhs' => 83, 'rhs' => 1 ),
  array( 'lhs' => 83, 'rhs' => 2 ),
  array( 'lhs' => 83, 'rhs' => 2 ),
  array( 'lhs' => 87, 'rhs' => 1 ),
  array( 'lhs' => 95, 'rhs' => 2 ),
  array( 'lhs' => 102, 'rhs' => 2 ),
  array( 'lhs' => 102, 'rhs' => 3 ),
  array( 'lhs' => 102, 'rhs' => 2 ),
  array( 'lhs' => 102, 'rhs' => 1 ),
  array( 'lhs' => 102, 'rhs' => 2 ),
  array( 'lhs' => 102, 'rhs' => 4 ),
  array( 'lhs' => 102, 'rhs' => 3 ),
  array( 'lhs' => 102, 'rhs' => 3 ),
  array( 'lhs' => 103, 'rhs' => 2 ),
  array( 'lhs' => 103, 'rhs' => 3 ),
  array( 'lhs' => 103, 'rhs' => 5 ),
  array( 'lhs' => 103, 'rhs' => 2 ),
  array( 'lhs' => 103, 'rhs' => 2 ),
  array( 'lhs' => 103, 'rhs' => 4 ),
  array( 'lhs' => 103, 'rhs' => 3 ),
  array( 'lhs' => 103, 'rhs' => 5 ),
  array( 'lhs' => 103, 'rhs' => 3 ),
  array( 'lhs' => 91, 'rhs' => 1 ),
  array( 'lhs' => 91, 'rhs' => 2 ),
  array( 'lhs' => 105, 'rhs' => 1 ),
  array( 'lhs' => 105, 'rhs' => 3 ),
  array( 'lhs' => 104, 'rhs' => 2 ),
  array( 'lhs' => 104, 'rhs' => 3 ),
  array( 'lhs' => 104, 'rhs' => 4 ),
  array( 'lhs' => 104, 'rhs' => 5 ),
  array( 'lhs' => 104, 'rhs' => 2 ),
  array( 'lhs' => 100, 'rhs' => 4 ),
  array( 'lhs' => 106, 'rhs' => 4 ),
  array( 'lhs' => 106, 'rhs' => 5 ),
  array( 'lhs' => 107, 'rhs' => 3 ),
  array( 'lhs' => 107, 'rhs' => 1 ),
  array( 'lhs' => 107, 'rhs' => 0 ),
  array( 'lhs' => 84, 'rhs' => 3 ),
  array( 'lhs' => 84, 'rhs' => 2 ),
  array( 'lhs' => 108, 'rhs' => 3 ),
  array( 'lhs' => 108, 'rhs' => 2 ),
  array( 'lhs' => 93, 'rhs' => 2 ),
  array( 'lhs' => 93, 'rhs' => 0 ),
  array( 'lhs' => 109, 'rhs' => 2 ),
  array( 'lhs' => 109, 'rhs' => 2 ),
  array( 'lhs' => 98, 'rhs' => 1 ),
  array( 'lhs' => 98, 'rhs' => 1 ),
  array( 'lhs' => 98, 'rhs' => 1 ),
  array( 'lhs' => 98, 'rhs' => 1 ),
  array( 'lhs' => 98, 'rhs' => 1 ),
  array( 'lhs' => 98, 'rhs' => 1 ),
  array( 'lhs' => 98, 'rhs' => 1 ),
  array( 'lhs' => 98, 'rhs' => 1 ),
  array( 'lhs' => 98, 'rhs' => 1 ),
  array( 'lhs' => 99, 'rhs' => 1 ),
  array( 'lhs' => 99, 'rhs' => 1 ),
  array( 'lhs' => 99, 'rhs' => 1 ),
  array( 'lhs' => 97, 'rhs' => 3 ),
  array( 'lhs' => 110, 'rhs' => 1 ),
  array( 'lhs' => 110, 'rhs' => 3 ),
  array( 'lhs' => 110, 'rhs' => 0 ),
  array( 'lhs' => 111, 'rhs' => 3 ),
  array( 'lhs' => 111, 'rhs' => 3 ),
  array( 'lhs' => 111, 'rhs' => 1 ),
  array( 'lhs' => 101, 'rhs' => 2 ),
  array( 'lhs' => 101, 'rhs' => 3 ),
  array( 'lhs' => 112, 'rhs' => 2 ),
  array( 'lhs' => 112, 'rhs' => 1 ),
  array( 'lhs' => 113, 'rhs' => 1 ),
  array( 'lhs' => 113, 'rhs' => 3 ),
  array( 'lhs' => 113, 'rhs' => 3 ),
  array( 'lhs' => 113, 'rhs' => 2 ),
  array( 'lhs' => 113, 'rhs' => 1 ),
  array( 'lhs' => 90, 'rhs' => 1 ),
  array( 'lhs' => 90, 'rhs' => 0 ),
    );

    public static $yyReduceMap = array(
        0 => 0,
        1 => 1,
        2 => 2,
        13 => 2,
        16 => 2,
        178 => 2,
        3 => 3,
        4 => 3,
        6 => 6,
        7 => 7,
        8 => 8,
        9 => 9,
        10 => 10,
        11 => 11,
        12 => 12,
        14 => 14,
        64 => 14,
        125 => 14,
        15 => 15,
        94 => 15,
        95 => 15,
        113 => 15,
        17 => 17,
        18 => 17,
        37 => 17,
        57 => 17,
        58 => 17,
        65 => 17,
        66 => 17,
        71 => 17,
        91 => 17,
        101 => 17,
        107 => 17,
        121 => 17,
        162 => 17,
        167 => 17,
        177 => 17,
        19 => 19,
        20 => 20,
        22 => 20,
        21 => 21,
        23 => 21,
        24 => 24,
        25 => 25,
        26 => 25,
        27 => 27,
        28 => 28,
        29 => 29,
        30 => 30,
        31 => 31,
        33 => 31,
        32 => 32,
        34 => 32,
        35 => 35,
        36 => 36,
        38 => 38,
        39 => 39,
        40 => 40,
        41 => 41,
        43 => 41,
        42 => 42,
        44 => 42,
        45 => 45,
        46 => 46,
        47 => 47,
        48 => 48,
        49 => 49,
        50 => 50,
        51 => 51,
        60 => 51,
        139 => 51,
        147 => 51,
        148 => 51,
        52 => 52,
        140 => 52,
        146 => 52,
        53 => 53,
        54 => 54,
        55 => 54,
        56 => 56,
        59 => 59,
        61 => 61,
        62 => 62,
        63 => 63,
        67 => 67,
        68 => 68,
        69 => 68,
        70 => 68,
        72 => 72,
        106 => 72,
        73 => 73,
        76 => 73,
        74 => 74,
        75 => 75,
        77 => 77,
        78 => 78,
        79 => 79,
        80 => 80,
        81 => 81,
        82 => 82,
        83 => 83,
        84 => 84,
        85 => 85,
        86 => 86,
        87 => 87,
        88 => 88,
        89 => 89,
        90 => 90,
        92 => 92,
        93 => 93,
        96 => 96,
        103 => 96,
        104 => 96,
        97 => 97,
        98 => 98,
        99 => 99,
        100 => 100,
        102 => 102,
        105 => 105,
        108 => 108,
        126 => 108,
        109 => 109,
        110 => 110,
        111 => 111,
        112 => 112,
        114 => 114,
        115 => 115,
        116 => 116,
        117 => 117,
        118 => 118,
        119 => 119,
        120 => 120,
        122 => 122,
        123 => 123,
        124 => 124,
        127 => 127,
        128 => 128,
        129 => 129,
        130 => 130,
        131 => 131,
        132 => 132,
        133 => 133,
        134 => 134,
        135 => 135,
        136 => 136,
        137 => 137,
        138 => 138,
        141 => 141,
        142 => 142,
        143 => 143,
        144 => 144,
        145 => 145,
        149 => 149,
        150 => 150,
        151 => 151,
        152 => 152,
        153 => 153,
        154 => 154,
        155 => 155,
        156 => 156,
        157 => 157,
        158 => 158,
        159 => 159,
        160 => 160,
        161 => 161,
        163 => 163,
        164 => 164,
        165 => 165,
        166 => 166,
        168 => 168,
        169 => 169,
        170 => 170,
        171 => 171,
        172 => 172,
        173 => 173,
        174 => 174,
        175 => 175,
        176 => 176,
    );
#line 135 "smarty_internal_templateparser.y"
    function yy_r0(){
    $this->_retvalue = $this->yystack[$this->yyidx + -1]->minor . $this->root_buffer->to_smarty_php();
    }
#line 2021 "smarty_internal_templateparser.php"
#line 139 "smarty_internal_templateparser.y"
    function yy_r1(){
    $this->strict_mode = true;
    $this->_retvalue = "/* strict mode */\n\$_smarty_tpl->strict_mode = true;\n";
    }
#line 2027 "smarty_internal_templateparser.php"
#line 143 "smarty_internal_templateparser.y"
    function yy_r2(){
    $this->_retvalue = '';
    }
#line 2032 "smarty_internal_templateparser.php"
#line 149 "smarty_internal_templateparser.y"
    function yy_r3(){
    if ($this->yystack[$this->yyidx + 0]->minor !== null) {
        $this->current_buffer->append_subtree($this->yystack[$this->yyidx + 0]->minor);
    }
    }
#line 2039 "smarty_internal_templateparser.php"
#line 166 "smarty_internal_templateparser.y"
    function yy_r6(){
    if ($this->compiler->has_code) {
        $tmp =''; foreach ($this->compiler->prefix_code as $code) {$tmp.=$code;} $this->compiler->prefix_code=array();
        $this->_retvalue = new _smarty_tag($this, $tmp.$this->yystack[$this->yyidx + -1]->minor);
    } else {
        $this->_retvalue = null;
    }
    $this->compiler->has_variable_string = false;
    $this->block_nesting_level = count($this->compiler->_tag_stack);
    }
#line 2051 "smarty_internal_templateparser.php"
#line 178 "smarty_internal_templateparser.y"
    function yy_r7(){
    $this->_retvalue = null;
    }
#line 2056 "smarty_internal_templateparser.php"
#line 183 "smarty_internal_templateparser.y"
    function yy_r8(){
    $this->_retvalue = new _smarty_text($this, $this->yystack[$this->yyidx + 0]->minor);
    }
#line 2061 "smarty_internal_templateparser.php"
#line 188 "smarty_internal_templateparser.y"
    function yy_r9(){
    if ($this->strip) {
        $this->_retvalue = new _smarty_text($this, self::stripString($this->yystack[$this->yyidx + 0]->minor));
    } else {
        $this->_retvalue = new _smarty_text($this, $this->yystack[$this->yyidx + 0]->minor);
    }
    }
#line 2070 "smarty_internal_templateparser.php"
#line 197 "smarty_internal_templateparser.y"
    function yy_r10(){
    $this->strip++;
    }
#line 2075 "smarty_internal_templateparser.php"
#line 201 "smarty_internal_templateparser.y"
    function yy_r11(){
    if (!$this->strip) {
        $this->compiler->trigger_template_error('Unbalanced {strip} tags');
    }
    $this->strip--;
    }
#line 2083 "smarty_internal_templateparser.php"
#line 208 "smarty_internal_templateparser.y"
    function yy_r12(){
    if ($this->strip) {
        SMARTY_INTERNAL_COMPILE_BLOCK::blockSource($this->compiler, self::stripString($this->yystack[$this->yyidx + 0]->minor));
    } else {
        SMARTY_INTERNAL_COMPILE_BLOCK::blockSource($this->compiler, $this->yystack[$this->yyidx + 0]->minor);
    }
    }
#line 2092 "smarty_internal_templateparser.php"
#line 220 "smarty_internal_templateparser.y"
    function yy_r14(){
    $this->_retvalue = $this->yystack[$this->yyidx + -1]->minor;
    }
#line 2097 "smarty_internal_templateparser.php"
#line 224 "smarty_internal_templateparser.y"
    function yy_r15(){
    $this->_retvalue = $this->yystack[$this->yyidx + -1]->minor . $this->yystack[$this->yyidx + 0]->minor;
    }
#line 2102 "smarty_internal_templateparser.php"
#line 232 "smarty_internal_templateparser.y"
    function yy_r17(){
    $this->_retvalue = $this->yystack[$this->yyidx + 0]->minor;
    }
#line 2107 "smarty_internal_templateparser.php"
#line 246 "smarty_internal_templateparser.y"
    function yy_r19(){
    $this->compiler->assert_no_enforced_modifiers($this->yystack[$this->yyidx + 0]->minor instanceof BrainyStaticWrapper);
    if ($this->yystack[$this->yyidx + 0]->minor instanceof BrainyStaticWrapper) {
        $this->yystack[$this->yyidx + 0]->minor = (string) $this->yystack[$this->yyidx + 0]->minor;
    }
    $this->_retvalue = $this->compiler->compileTag('private_print_expression',array(),array('value'=>$this->yystack[$this->yyidx + 0]->minor));
    }
#line 2116 "smarty_internal_templateparser.php"
#line 254 "smarty_internal_templateparser.y"
    function yy_r20(){
    $this->compiler->assert_expected_modifier($this->yystack[$this->yyidx + -1]->minor, $this->yystack[$this->yyidx + -2]->minor instanceof BrainyStaticWrapper);
    if ($this->yystack[$this->yyidx + -2]->minor instanceof BrainyStaticWrapper) {
        $this->yystack[$this->yyidx + -2]->minor = (string) $this->yystack[$this->yyidx + -2]->minor;
    }
    $this->_retvalue = $this->compiler->compileTag('private_print_expression',$this->yystack[$this->yyidx + 0]->minor,array('value'=>$this->yystack[$this->yyidx + -2]->minor, 'modifierlist'=>$this->yystack[$this->yyidx + -1]->minor));
    }
#line 2125 "smarty_internal_templateparser.php"
#line 262 "smarty_internal_templateparser.y"
    function yy_r21(){
    $this->compiler->assert_no_enforced_modifiers($this->yystack[$this->yyidx + -1]->minor instanceof BrainyStaticWrapper);
    if ($this->yystack[$this->yyidx + -1]->minor instanceof BrainyStaticWrapper) {
        $this->yystack[$this->yyidx + -1]->minor = (string) $this->yystack[$this->yyidx + -1]->minor;
    }
    $this->_retvalue = $this->compiler->compileTag('private_print_expression',$this->yystack[$this->yyidx + 0]->minor,array('value'=>$this->yystack[$this->yyidx + -1]->minor));
    }
#line 2134 "smarty_internal_templateparser.php"
#line 289 "smarty_internal_templateparser.y"
    function yy_r24(){
    $this->_retvalue = $this->yystack[$this->yyidx + -2]->minor . ' = (' . $this->yystack[$this->yyidx + 0]->minor . ');';
    }
#line 2139 "smarty_internal_templateparser.php"
#line 293 "smarty_internal_templateparser.y"
    function yy_r25(){
    $this->_retvalue = $this->compiler->compileTag('assign',array(array('value'=>$this->yystack[$this->yyidx + 0]->minor),array('var'=>"'".$this->yystack[$this->yyidx + -2]->minor."'")));
    }
#line 2144 "smarty_internal_templateparser.php"
#line 301 "smarty_internal_templateparser.y"
    function yy_r27(){
    $this->compiler->assert_is_not_strict('Passing attributes in an assignment without using {assign} is not supported in strict mode', $this);
    $this->_retvalue = $this->compiler->compileTag('assign',array_merge(array(array('value'=>$this->yystack[$this->yyidx + -1]->minor),array('var'=>"'".$this->yystack[$this->yyidx + -3]->minor."'")),$this->yystack[$this->yyidx + 0]->minor));
    }
#line 2150 "smarty_internal_templateparser.php"
#line 307 "smarty_internal_templateparser.y"
    function yy_r28(){
    $this->_retvalue = $this->compiler->compileTag($this->yystack[$this->yyidx + -1]->minor,$this->yystack[$this->yyidx + 0]->minor);
    }
#line 2155 "smarty_internal_templateparser.php"
#line 311 "smarty_internal_templateparser.y"
    function yy_r29(){
    $this->_retvalue = $this->compiler->compileTag($this->yystack[$this->yyidx + 0]->minor,array());
    }
#line 2160 "smarty_internal_templateparser.php"
#line 316 "smarty_internal_templateparser.y"
    function yy_r30(){
    $this->_retvalue = "ob_start();\n".$this->compiler->compileTag($this->yystack[$this->yyidx + -2]->minor,$this->yystack[$this->yyidx + 0]->minor).'echo ';
    $this->_retvalue .= $this->compiler->compileTag('private_modifier',array(),array('modifierlist'=>$this->yystack[$this->yyidx + -1]->minor,'value'=>'ob_get_clean()')) . ";\n";
    }
#line 2166 "smarty_internal_templateparser.php"
#line 323 "smarty_internal_templateparser.y"
    function yy_r31(){
    $tag = trim(substr($this->yystack[$this->yyidx + -1]->minor,$this->lex->ldel_length));
    $this->_retvalue = $this->compiler->compileTag(($tag == 'else if')? 'elseif' : $tag,array(),array('if condition'=>$this->yystack[$this->yyidx + 0]->minor));
    }
#line 2172 "smarty_internal_templateparser.php"
#line 328 "smarty_internal_templateparser.y"
    function yy_r32(){
    $tag = trim(substr($this->yystack[$this->yyidx + -2]->minor,$this->lex->ldel_length));
    $this->_retvalue = $this->compiler->compileTag(($tag == 'else if')? 'elseif' : $tag,$this->yystack[$this->yyidx + 0]->minor,array('if condition'=>$this->yystack[$this->yyidx + -1]->minor));
    }
#line 2178 "smarty_internal_templateparser.php"
#line 344 "smarty_internal_templateparser.y"
    function yy_r35(){
    $this->_retvalue = $this->compiler->compileTag('for',array_merge($this->yystack[$this->yyidx + 0]->minor,array(array('start'=>$this->yystack[$this->yyidx + -9]->minor),array('ifexp'=>$this->yystack[$this->yyidx + -6]->minor),array('var'=>$this->yystack[$this->yyidx + -2]->minor),array('step'=>$this->yystack[$this->yyidx + -1]->minor))),1);
    }
#line 2183 "smarty_internal_templateparser.php"
#line 348 "smarty_internal_templateparser.y"
    function yy_r36(){
    $this->_retvalue = '='.$this->yystack[$this->yyidx + 0]->minor;
    }
#line 2188 "smarty_internal_templateparser.php"
#line 356 "smarty_internal_templateparser.y"
    function yy_r38(){
    $this->_retvalue = $this->compiler->compileTag('for',array_merge($this->yystack[$this->yyidx + 0]->minor,array(array('start'=>$this->yystack[$this->yyidx + -3]->minor),array('to'=>$this->yystack[$this->yyidx + -1]->minor))),0);
    }
#line 2193 "smarty_internal_templateparser.php"
#line 360 "smarty_internal_templateparser.y"
    function yy_r39(){
    $this->_retvalue = $this->compiler->compileTag('for',array_merge($this->yystack[$this->yyidx + 0]->minor,array(array('start'=>$this->yystack[$this->yyidx + -5]->minor),array('to'=>$this->yystack[$this->yyidx + -3]->minor),array('step'=>$this->yystack[$this->yyidx + -1]->minor))),0);
    }
#line 2198 "smarty_internal_templateparser.php"
#line 365 "smarty_internal_templateparser.y"
    function yy_r40(){
    $this->_retvalue = $this->compiler->compileTag('foreach',$this->yystack[$this->yyidx + 0]->minor);
    }
#line 2203 "smarty_internal_templateparser.php"
#line 370 "smarty_internal_templateparser.y"
    function yy_r41(){
    $this->_retvalue = $this->compiler->compileTag('foreach',array_merge($this->yystack[$this->yyidx + 0]->minor,array(array('from'=>$this->yystack[$this->yyidx + -4]->minor),array('item'=>$this->yystack[$this->yyidx + -1]->minor))));
    }
#line 2208 "smarty_internal_templateparser.php"
#line 374 "smarty_internal_templateparser.y"
    function yy_r42(){
    $this->_retvalue = $this->compiler->compileTag('foreach',array_merge($this->yystack[$this->yyidx + 0]->minor,array(array('from'=>$this->yystack[$this->yyidx + -7]->minor),array('item'=>$this->yystack[$this->yyidx + -1]->minor),array('key'=>$this->yystack[$this->yyidx + -4]->minor))));
    }
#line 2213 "smarty_internal_templateparser.php"
#line 387 "smarty_internal_templateparser.y"
    function yy_r45(){
    $this->_retvalue = $this->compiler->compileTag('setfilter',array(),array('modifier_list'=>array(array_merge(array($this->yystack[$this->yyidx + -1]->minor),$this->yystack[$this->yyidx + 0]->minor))));
    }
#line 2218 "smarty_internal_templateparser.php"
#line 391 "smarty_internal_templateparser.y"
    function yy_r46(){
    $this->_retvalue = $this->compiler->compileTag('setfilter',array(),array('modifier_list'=>array_merge(array(array_merge(array($this->yystack[$this->yyidx + -2]->minor),$this->yystack[$this->yyidx + -1]->minor)),$this->yystack[$this->yyidx + 0]->minor)));
    }
#line 2223 "smarty_internal_templateparser.php"
#line 396 "smarty_internal_templateparser.y"
    function yy_r47(){
    $j = strrpos($this->yystack[$this->yyidx + 0]->minor,'.');
    if ($this->yystack[$this->yyidx + 0]->minor[$j+1] == 'c') {
        // {$smarty.block.child}
        $this->_retvalue = SMARTY_INTERNAL_COMPILE_BLOCK::compileChildBlock($this->compiler);
    } else {
        // {$smarty.block.parent}
        $this->_retvalue = SMARTY_INTERNAL_COMPILE_BLOCK::compileParentBlock($this->compiler);
    }
    }
#line 2235 "smarty_internal_templateparser.php"
#line 409 "smarty_internal_templateparser.y"
    function yy_r48(){
    $this->_retvalue = $this->compiler->compileTag($this->yystack[$this->yyidx + 0]->minor.'close',array());
    }
#line 2240 "smarty_internal_templateparser.php"
#line 413 "smarty_internal_templateparser.y"
    function yy_r49(){
    $this->_retvalue = $this->compiler->compileTag($this->yystack[$this->yyidx + -1]->minor.'close',array(),array('modifier_list'=>$this->yystack[$this->yyidx + 0]->minor));
    }
#line 2245 "smarty_internal_templateparser.php"
#line 421 "smarty_internal_templateparser.y"
    function yy_r50(){
    $this->_retvalue = $this->yystack[$this->yyidx + -1]->minor;
    $this->_retvalue[] = $this->yystack[$this->yyidx + 0]->minor;
    }
#line 2251 "smarty_internal_templateparser.php"
#line 427 "smarty_internal_templateparser.y"
    function yy_r51(){
    $this->_retvalue = array($this->yystack[$this->yyidx + 0]->minor);
    }
#line 2256 "smarty_internal_templateparser.php"
#line 432 "smarty_internal_templateparser.y"
    function yy_r52(){
    $this->_retvalue = array();
    }
#line 2261 "smarty_internal_templateparser.php"
#line 437 "smarty_internal_templateparser.y"
    function yy_r53(){
    if (preg_match('~^true$~i', $this->yystack[$this->yyidx + 0]->minor)) {
        $this->_retvalue = array($this->yystack[$this->yyidx + -2]->minor=>'true');
    } elseif (preg_match('~^false$~i', $this->yystack[$this->yyidx + 0]->minor)) {
        $this->_retvalue = array($this->yystack[$this->yyidx + -2]->minor=>'false');
    } elseif (preg_match('~^null$~i', $this->yystack[$this->yyidx + 0]->minor)) {
        $this->_retvalue = array($this->yystack[$this->yyidx + -2]->minor=>'null');
    } else {
        $this->_retvalue = array($this->yystack[$this->yyidx + -2]->minor=>"'".$this->yystack[$this->yyidx + 0]->minor."'");
    }
    }
#line 2274 "smarty_internal_templateparser.php"
#line 449 "smarty_internal_templateparser.y"
    function yy_r54(){
    $this->_retvalue = array(trim($this->yystack[$this->yyidx + -1]->minor," =\n\r\t")=>$this->yystack[$this->yyidx + 0]->minor);
    }
#line 2279 "smarty_internal_templateparser.php"
#line 457 "smarty_internal_templateparser.y"
    function yy_r56(){
    $this->_retvalue = "'".$this->yystack[$this->yyidx + 0]->minor."'";
    }
#line 2284 "smarty_internal_templateparser.php"
#line 469 "smarty_internal_templateparser.y"
    function yy_r59(){
    $this->_retvalue = array($this->yystack[$this->yyidx + -2]->minor=>$this->yystack[$this->yyidx + 0]->minor);
    }
#line 2289 "smarty_internal_templateparser.php"
#line 482 "smarty_internal_templateparser.y"
    function yy_r61(){
    $this->yystack[$this->yyidx + -2]->minor[]=$this->yystack[$this->yyidx + 0]->minor;
    $this->_retvalue = $this->yystack[$this->yyidx + -2]->minor;
    }
#line 2295 "smarty_internal_templateparser.php"
#line 487 "smarty_internal_templateparser.y"
    function yy_r62(){
    $this->compiler->assert_is_not_strict('Variable variable assignment is not supported in strict mode', $this);
    $this->_retvalue = array('var' => $this->yystack[$this->yyidx + -2]->minor, 'value'=>$this->yystack[$this->yyidx + 0]->minor);
    }
#line 2301 "smarty_internal_templateparser.php"
#line 492 "smarty_internal_templateparser.y"
    function yy_r63(){
    $this->_retvalue = array('var' => $this->yystack[$this->yyidx + -2]->minor, 'value'=>$this->yystack[$this->yyidx + 0]->minor);
    }
#line 2306 "smarty_internal_templateparser.php"
#line 516 "smarty_internal_templateparser.y"
    function yy_r67(){
    $this->compiler->assert_is_not_strict('Stream access is not supported in strict mode', $this);
    $this->_retvalue = '$_smarty_tpl->getStreamVariable(\''. $this->yystack[$this->yyidx + -2]->minor .'://'. $this->yystack[$this->yyidx + 0]->minor . '\')';
    }
#line 2312 "smarty_internal_templateparser.php"
#line 522 "smarty_internal_templateparser.y"
    function yy_r68(){
    $this->_retvalue = BrainyStaticWrapper::static_if_all($this->yystack[$this->yyidx + -2]->minor . trim($this->yystack[$this->yyidx + -1]->minor) . $this->yystack[$this->yyidx + 0]->minor, array($this->yystack[$this->yyidx + -2]->minor, $this->yystack[$this->yyidx + 0]->minor));
    }
#line 2317 "smarty_internal_templateparser.php"
#line 541 "smarty_internal_templateparser.y"
    function yy_r72(){
    $this->_retvalue = $this->compiler->compileTag('private_modifier',array(),array('value'=>$this->yystack[$this->yyidx + -1]->minor,'modifierlist'=>$this->yystack[$this->yyidx + 0]->minor));
    }
#line 2322 "smarty_internal_templateparser.php"
#line 547 "smarty_internal_templateparser.y"
    function yy_r73(){
    $this->_retvalue = new BrainyStaticWrapper($this->yystack[$this->yyidx + -2]->minor.$this->yystack[$this->yyidx + -1]->minor.$this->yystack[$this->yyidx + 0]->minor);
    }
#line 2327 "smarty_internal_templateparser.php"
#line 551 "smarty_internal_templateparser.y"
    function yy_r74(){
    $this->_retvalue = new BrainyStaticWrapper('in_array('.$this->yystack[$this->yyidx + -2]->minor.','.$this->yystack[$this->yyidx + 0]->minor.')');
    }
#line 2332 "smarty_internal_templateparser.php"
#line 555 "smarty_internal_templateparser.y"
    function yy_r75(){
    $this->_retvalue = new BrainyStaticWrapper('in_array('.$this->yystack[$this->yyidx + -2]->minor.',(array)'.$this->yystack[$this->yyidx + 0]->minor.')');
    }
#line 2337 "smarty_internal_templateparser.php"
#line 563 "smarty_internal_templateparser.y"
    function yy_r77(){
    $this->_retvalue = new BrainyStaticWrapper('!('.$this->yystack[$this->yyidx + -2]->minor.' % '.$this->yystack[$this->yyidx + 0]->minor.')');
    }
#line 2342 "smarty_internal_templateparser.php"
#line 567 "smarty_internal_templateparser.y"
    function yy_r78(){
    $this->compiler->assert_is_not_strict('`is not div by` is not supported in strict mode', $this);
    $this->_retvalue = new BrainyStaticWrapper('('.$this->yystack[$this->yyidx + -2]->minor.' % '.$this->yystack[$this->yyidx + 0]->minor.')');
    }
#line 2348 "smarty_internal_templateparser.php"
#line 572 "smarty_internal_templateparser.y"
    function yy_r79(){
    $this->_retvalue = new BrainyStaticWrapper('!(1 & '.$this->yystack[$this->yyidx + -1]->minor.')');
    }
#line 2353 "smarty_internal_templateparser.php"
#line 576 "smarty_internal_templateparser.y"
    function yy_r80(){
    $this->compiler->assert_is_not_strict('`is not even` is not supported in strict mode', $this);
    $this->_retvalue = new BrainyStaticWrapper('(1 & '.$this->yystack[$this->yyidx + -1]->minor.')');
    }
#line 2359 "smarty_internal_templateparser.php"
#line 581 "smarty_internal_templateparser.y"
    function yy_r81(){
    $this->compiler->assert_is_not_strict('`is even by` is not supported in strict mode', $this);
    $this->_retvalue = new BrainyStaticWrapper('!(1 & '.$this->yystack[$this->yyidx + -2]->minor.' / '.$this->yystack[$this->yyidx + 0]->minor.')');
    }
#line 2365 "smarty_internal_templateparser.php"
#line 586 "smarty_internal_templateparser.y"
    function yy_r82(){
    $this->compiler->assert_is_not_strict('`is not even by` is not supported in strict mode', $this);
    $this->_retvalue = new BrainyStaticWrapper('(1 & '.$this->yystack[$this->yyidx + -2]->minor.' / '.$this->yystack[$this->yyidx + 0]->minor.')');
    }
#line 2371 "smarty_internal_templateparser.php"
#line 591 "smarty_internal_templateparser.y"
    function yy_r83(){
    $this->_retvalue = new BrainyStaticWrapper('(1 & '.$this->yystack[$this->yyidx + -1]->minor.')');
    }
#line 2376 "smarty_internal_templateparser.php"
#line 595 "smarty_internal_templateparser.y"
    function yy_r84(){
    $this->compiler->assert_is_not_strict('`is not odd` is not supported in strict mode', $this);
    $this->_retvalue = new BrainyStaticWrapper('!(1 & '.$this->yystack[$this->yyidx + -1]->minor.')');
    }
#line 2382 "smarty_internal_templateparser.php"
#line 600 "smarty_internal_templateparser.y"
    function yy_r85(){
    $this->compiler->assert_is_not_strict('`is odd by` is not supported in strict mode', $this);
    $this->_retvalue = new BrainyStaticWrapper('(1 & '.$this->yystack[$this->yyidx + -2]->minor.' / '.$this->yystack[$this->yyidx + 0]->minor.')');
    }
#line 2388 "smarty_internal_templateparser.php"
#line 605 "smarty_internal_templateparser.y"
    function yy_r86(){
    $this->compiler->assert_is_not_strict('`is not odd by` is not supported in strict mode', $this);
    $this->_retvalue = new BrainyStaticWrapper('!(1 & '.$this->yystack[$this->yyidx + -2]->minor.' / '.$this->yystack[$this->yyidx + 0]->minor.')');
    }
#line 2394 "smarty_internal_templateparser.php"
#line 610 "smarty_internal_templateparser.y"
    function yy_r87(){
    $this->compiler->assert_is_not_strict('`instanceof` is not supported in strict mode', $this);
    $this->_retvalue = new BrainyStaticWrapper($this->yystack[$this->yyidx + -2]->minor.$this->yystack[$this->yyidx + -1]->minor.$this->yystack[$this->yyidx + 0]->minor);
    }
#line 2400 "smarty_internal_templateparser.php"
#line 615 "smarty_internal_templateparser.y"
    function yy_r88(){
    $this->compiler->assert_is_not_strict('`instanceof` is not supported in strict mode', $this);
    self::$prefix_number++;
    $this->compiler->prefix_code[] = '$_tmp'.self::$prefix_number.'='.$this->yystack[$this->yyidx + 0]->minor.";\n";
    $this->_retvalue = new BrainyStaticWrapper($this->yystack[$this->yyidx + -2]->minor.$this->yystack[$this->yyidx + -1]->minor.'$_tmp'.self::$prefix_number);
    }
#line 2408 "smarty_internal_templateparser.php"
#line 625 "smarty_internal_templateparser.y"
    function yy_r89(){
    $this->_retvalue = $this->yystack[$this->yyidx + -6]->minor.' ? '. $this->compileVariable("'".$this->yystack[$this->yyidx + -2]->minor."'") . ' : '.$this->yystack[$this->yyidx + 0]->minor;
    }
#line 2413 "smarty_internal_templateparser.php"
#line 629 "smarty_internal_templateparser.y"
    function yy_r90(){
    $this->_retvalue = $this->yystack[$this->yyidx + -5]->minor.' ? '.$this->yystack[$this->yyidx + -2]->minor.' : '.$this->yystack[$this->yyidx + 0]->minor;
    }
#line 2418 "smarty_internal_templateparser.php"
#line 639 "smarty_internal_templateparser.y"
    function yy_r92(){
    $this->_retvalue = BrainyStaticWrapper::static_concat($this->yystack[$this->yyidx + -1]->minor, $this->yystack[$this->yyidx + 0]->minor);
    }
#line 2423 "smarty_internal_templateparser.php"
#line 644 "smarty_internal_templateparser.y"
    function yy_r93(){
    $this->_retvalue = BrainyStaticWrapper::static_concat('!', $this->yystack[$this->yyidx + 0]->minor);
    }
#line 2428 "smarty_internal_templateparser.php"
#line 657 "smarty_internal_templateparser.y"
    function yy_r96(){
    $this->_retvalue = new BrainyStaticWrapper($this->yystack[$this->yyidx + 0]->minor);
    }
#line 2433 "smarty_internal_templateparser.php"
#line 661 "smarty_internal_templateparser.y"
    function yy_r97(){
    $this->_retvalue = new BrainyStaticWrapper($this->yystack[$this->yyidx + -2]->minor.'.'.$this->yystack[$this->yyidx + 0]->minor);
    }
#line 2438 "smarty_internal_templateparser.php"
#line 665 "smarty_internal_templateparser.y"
    function yy_r98(){
    $this->_retvalue = new BrainyStaticWrapper($this->yystack[$this->yyidx + -1]->minor.'.');
    }
#line 2443 "smarty_internal_templateparser.php"
#line 669 "smarty_internal_templateparser.y"
    function yy_r99(){
    $this->_retvalue = new BrainyStaticWrapper('.'.$this->yystack[$this->yyidx + 0]->minor);
    }
#line 2448 "smarty_internal_templateparser.php"
#line 674 "smarty_internal_templateparser.y"
    function yy_r100(){
    if (preg_match('~^true$~i', $this->yystack[$this->yyidx + 0]->minor)) {
        $this->_retvalue = new BrainyStaticWrapper('true');
    } elseif (preg_match('~^false$~i', $this->yystack[$this->yyidx + 0]->minor)) {
        $this->_retvalue = new BrainyStaticWrapper('false');
    } elseif (preg_match('~^null$~i', $this->yystack[$this->yyidx + 0]->minor)) {
        $this->_retvalue = new BrainyStaticWrapper('null');
    } else {
        $this->_retvalue = new BrainyStaticWrapper(var_export($this->yystack[$this->yyidx + 0]->minor, true));
    }
    }
#line 2461 "smarty_internal_templateparser.php"
#line 692 "smarty_internal_templateparser.y"
    function yy_r102(){
    $this->_retvalue = BrainyStaticWrapper::static_if_all("(". $this->yystack[$this->yyidx + -1]->minor .")", array($this->yystack[$this->yyidx + -1]->minor));
    }
#line 2466 "smarty_internal_templateparser.php"
#line 708 "smarty_internal_templateparser.y"
    function yy_r105(){
    self::$prefix_number++;
    $this->compiler->prefix_code[] = 'ob_start();'.$this->yystack[$this->yyidx + -1]->minor.'$_tmp'.self::$prefix_number.'=ob_get_clean();';
    $this->_retvalue = '$_tmp'.self::$prefix_number;
    }
#line 2473 "smarty_internal_templateparser.php"
#line 727 "smarty_internal_templateparser.y"
    function yy_r108(){
    $this->compiler->assert_is_not_strict('Variable variables are not supported in strict mode', $this);
    $this->_retvalue = $this->yystack[$this->yyidx + 0]->minor;
    }
#line 2479 "smarty_internal_templateparser.php"
#line 732 "smarty_internal_templateparser.y"
    function yy_r109(){
    if ($this->yystack[$this->yyidx + 0]->minor === '[]') {
        $this->_retvalue = $this->yystack[$this->yyidx + -1]->minor . $this->yystack[$this->yyidx + 0]->minor;
    } else {
        $this->_retvalue = $this->compileSafeLookupWithBase($this->yystack[$this->yyidx + -1]->minor, $this->yystack[$this->yyidx + 0]->minor);
    }
    }
#line 2488 "smarty_internal_templateparser.php"
#line 741 "smarty_internal_templateparser.y"
    function yy_r110(){
    if ($this->yystack[$this->yyidx + -2]->minor == '\'smarty\'') {
        $this->_retvalue = $this->compiler->compileTag('private_special_variable', array(), $this->yystack[$this->yyidx + -1]->minor, $this->yystack[$this->yyidx + 0]->minor);
    } else {
        $this->_retvalue = $this->compileSafeLookupWithBase($this->compileVariable($this->yystack[$this->yyidx + -2]->minor), $this->yystack[$this->yyidx + -1]->minor);
        $this->_retvalue = $this->compileSafeLookupWithBase($this->_retvalue, $this->yystack[$this->yyidx + 0]->minor);
    }
    }
#line 2498 "smarty_internal_templateparser.php"
#line 750 "smarty_internal_templateparser.y"
    function yy_r111(){
    if ($this->yystack[$this->yyidx + -1]->minor == '\'smarty\'') {
        $this->_retvalue = $this->compiler->compileTag('private_special_variable', array(), $this->yystack[$this->yyidx + 0]->minor);
    } elseif ($this->yystack[$this->yyidx + 0]->minor === '[]') {
        $this->_retvalue = $this->compileVariable($this->yystack[$this->yyidx + -1]->minor) . $this->yystack[$this->yyidx + 0]->minor;
    } else {
        $this->_retvalue = $this->compileSafeLookupWithBase($this->compileVariable($this->yystack[$this->yyidx + -1]->minor), $this->yystack[$this->yyidx + 0]->minor);
    }
    }
#line 2509 "smarty_internal_templateparser.php"
#line 760 "smarty_internal_templateparser.y"
    function yy_r112(){
    $this->_retvalue = $this->compileVariable($this->yystack[$this->yyidx + 0]->minor);
    }
#line 2514 "smarty_internal_templateparser.php"
#line 768 "smarty_internal_templateparser.y"
    function yy_r114(){
    $this->_retvalue = $this->compileVariable($this->yystack[$this->yyidx + -2]->minor, $this->yystack[$this->yyidx + 0]->minor);
    }
#line 2519 "smarty_internal_templateparser.php"
#line 773 "smarty_internal_templateparser.y"
    function yy_r115(){
    $this->_retvalue = '$_smarty_tpl->getConfigVariable(\''. $this->yystack[$this->yyidx + -1]->minor .'\')';
    }
#line 2524 "smarty_internal_templateparser.php"
#line 777 "smarty_internal_templateparser.y"
    function yy_r116(){
    $this->_retvalue = '$_smarty_tpl->getConfigVariable('. $this->yystack[$this->yyidx + -1]->minor .')';
    }
#line 2529 "smarty_internal_templateparser.php"
#line 781 "smarty_internal_templateparser.y"
    function yy_r117(){
    $this->_retvalue = '[]';
    }
#line 2534 "smarty_internal_templateparser.php"
#line 787 "smarty_internal_templateparser.y"
    function yy_r118(){
    $this->compiler->assert_is_not_strict('Variable indicies with dot syntax is not supported in strict mode', $this);
    $this->_retvalue = $this->compileVariable($this->yystack[$this->yyidx + 0]->minor);
    }
#line 2540 "smarty_internal_templateparser.php"
#line 792 "smarty_internal_templateparser.y"
    function yy_r119(){
    $this->compiler->assert_is_not_strict('Variable indicies with dot syntax is not supported in strict mode', $this);
    $this->_retvalue = $this->compileVariable($this->yystack[$this->yyidx + -2]->minor).'->'.$this->yystack[$this->yyidx + 0]->minor;
    }
#line 2546 "smarty_internal_templateparser.php"
#line 797 "smarty_internal_templateparser.y"
    function yy_r120(){
    $this->_retvalue = "'". $this->yystack[$this->yyidx + 0]->minor ."'";
    }
#line 2551 "smarty_internal_templateparser.php"
#line 805 "smarty_internal_templateparser.y"
    function yy_r122(){
    $this->compiler->assert_is_not_strict('Dot syntax with expressions is not supported in strict mode', $this);
    $this->_retvalue = $this->yystack[$this->yyidx + -1]->minor;
    }
#line 2557 "smarty_internal_templateparser.php"
#line 811 "smarty_internal_templateparser.y"
    function yy_r123(){
    $this->compiler->assert_is_not_strict('Section tags are not supported in strict mode', $this);
    $this->_retvalue = $this->compiler->compileTag('private_special_variable', array(), '\'section\'', '\'' . $this->yystack[$this->yyidx + -1]->minor . '\'') . '[\'index\']';
    }
#line 2563 "smarty_internal_templateparser.php"
#line 816 "smarty_internal_templateparser.y"
    function yy_r124(){
    $this->compiler->assert_is_not_strict('Section tags are not supported in strict mode', $this);
    $this->_retvalue = $this->compiler->compileTag('private_special_variable', array(), '\'section\'', '\'' . $this->yystack[$this->yyidx + -3]->minor . '\']') . '[\''.$this->yystack[$this->yyidx + -1]->minor.'\']';
    }
#line 2569 "smarty_internal_templateparser.php"
#line 835 "smarty_internal_templateparser.y"
    function yy_r127(){
    $this->compiler->assert_is_not_strict('Variable variables are not supported in strict mode', $this);
    $this->_retvalue = $this->yystack[$this->yyidx + -1]->minor.'.'.$this->yystack[$this->yyidx + 0]->minor;
    }
#line 2575 "smarty_internal_templateparser.php"
#line 841 "smarty_internal_templateparser.y"
    function yy_r128(){
    $this->_retvalue = '\''.$this->yystack[$this->yyidx + 0]->minor.'\'';
    }
#line 2580 "smarty_internal_templateparser.php"
#line 846 "smarty_internal_templateparser.y"
    function yy_r129(){
    $this->_retvalue = '('.$this->yystack[$this->yyidx + -1]->minor.')';
    }
#line 2585 "smarty_internal_templateparser.php"
#line 855 "smarty_internal_templateparser.y"
    function yy_r130(){
    if ($this->security && substr($this->yystack[$this->yyidx + 0]->minor, 0, 1) == '_') {
        $this->compiler->trigger_template_error(self::Err1);
    }
    $this->_retvalue = '->'.$this->yystack[$this->yyidx + 0]->minor;
    }
#line 2593 "smarty_internal_templateparser.php"
#line 862 "smarty_internal_templateparser.y"
    function yy_r131(){
    $this->compiler->assert_is_not_strict('Variable method calls are not supported in strict mode', $this);
    if ($this->security) {
        $this->compiler->trigger_template_error (self::Err2);
    }
    $this->_retvalue = '->{'.$this->compileVariable($this->yystack[$this->yyidx + 0]->minor).'}';
    }
#line 2602 "smarty_internal_templateparser.php"
#line 870 "smarty_internal_templateparser.y"
    function yy_r132(){
    $this->compiler->assert_is_not_strict('Variable method calls are not supported in strict mode', $this);
    if ($this->security) {
        $this->compiler->trigger_template_error (self::Err2);
    }
    $this->_retvalue = '->{'.$this->yystack[$this->yyidx + -1]->minor.'}';
    }
#line 2611 "smarty_internal_templateparser.php"
#line 878 "smarty_internal_templateparser.y"
    function yy_r133(){
    $this->compiler->assert_is_not_strict('Variable method calls are not supported in strict mode', $this);
    if ($this->security) {
        $this->compiler->trigger_template_error (self::Err2);
    }
    $this->_retvalue = '->{\''.$this->yystack[$this->yyidx + -3]->minor.'\'.'.$this->yystack[$this->yyidx + -1]->minor.'}';
    }
#line 2620 "smarty_internal_templateparser.php"
#line 887 "smarty_internal_templateparser.y"
    function yy_r134(){
    $this->_retvalue = '->'.$this->yystack[$this->yyidx + 0]->minor;
    }
#line 2625 "smarty_internal_templateparser.php"
#line 895 "smarty_internal_templateparser.y"
    function yy_r135(){
    if (!$this->security || $this->smarty->security_policy->isTrustedPhpFunction($this->yystack[$this->yyidx + -3]->minor, $this->compiler)) {
        if (strcasecmp($this->yystack[$this->yyidx + -3]->minor, 'isset') === 0 || strcasecmp($this->yystack[$this->yyidx + -3]->minor, 'empty') === 0 || strcasecmp($this->yystack[$this->yyidx + -3]->minor, 'array') === 0 || is_callable($this->yystack[$this->yyidx + -3]->minor)) {
            $func_name = strtolower($this->yystack[$this->yyidx + -3]->minor);

            $is_language_construct = $func_name === 'isset' || $func_name === 'empty';
            $combined_params = array();
            foreach ($this->yystack[$this->yyidx + -1]->minor as $param) {
                if ($is_language_construct && $param instanceof BrainySafeLookupWrapper) {
                    $combined_params[] = $param->getUnsafe();
                    continue;
                }
                $combined_params[] = $param;
            }
            $par = implode(',', $combined_params);

            if ($func_name == 'isset') {
                if (count($combined_params) == 0) {
                    $this->compiler->trigger_template_error('Illegal number of paramer in "isset()"');
                }
                if (strncasecmp($par, '$_smarty_tpl->getConfigVariable', strlen('$_smarty_tpl->getConfigVariable')) === 0) {
                    $this->compiler->assert_is_not_strict('isset() is not allowed on config variables', $this);
                    self::$prefix_number++;
                    $this->compiler->prefix_code[] = '$_tmp'.self::$prefix_number.'='.str_replace(')',', false)',$par).";\n";
                    $isset_par = '$_tmp'.self::$prefix_number;
                } else {
                    $isset_par=str_replace("')->value","',null,true,false)->value",$par);
                }
                $this->_retvalue = $this->yystack[$this->yyidx + -3]->minor . "(". $isset_par .")";

            } elseif (in_array($func_name, array('empty', 'reset', 'current', 'end', 'prev', 'next'))) {

                if ($func_name !== 'empty') {
                    $this->compiler->assert_is_not_strict($func_name . ' is not allowed in strict mode', $this);
                }

                if (count($combined_params) != 1) {
                    $this->compiler->trigger_template_error('Illegal number of paramer in "empty()"');
                }
                if ($func_name == 'empty') {
                    $this->_retvalue = $func_name.'('.str_replace("')->value","',null,true,false)->value",$combined_params[0]).')';
                } else {
                    $this->_retvalue = $func_name.'('.$combined_params[0].')';
                }
            } else {
                $this->_retvalue = $this->yystack[$this->yyidx + -3]->minor . "(". $par .")";
            }
        } else {
            $this->compiler->trigger_template_error("unknown function \"" . $this->yystack[$this->yyidx + -3]->minor . "\"");
        }
    }
    }
#line 2679 "smarty_internal_templateparser.php"
#line 951 "smarty_internal_templateparser.y"
    function yy_r136(){
    if ($this->security && substr($this->yystack[$this->yyidx + -3]->minor,0,1) == '_') {
        $this->compiler->trigger_template_error(self::Err1);
    }
    $this->_retvalue = $this->yystack[$this->yyidx + -3]->minor . "(". implode(',',$this->yystack[$this->yyidx + -1]->minor) .")";
    }
#line 2687 "smarty_internal_templateparser.php"
#line 958 "smarty_internal_templateparser.y"
    function yy_r137(){
    if ($this->security) {
        $this->compiler->trigger_template_error(self::Err2);
    }
    self::$prefix_number++;
    $this->compiler->prefix_code[] = '$_tmp'.self::$prefix_number.'='.$this->compileVariable("'".$this->yystack[$this->yyidx + -3]->minor."'").';';
    $this->_retvalue = '$_tmp'.self::$prefix_number.'('. implode(',',$this->yystack[$this->yyidx + -1]->minor) .')';
    }
#line 2697 "smarty_internal_templateparser.php"
#line 969 "smarty_internal_templateparser.y"
    function yy_r138(){
    $this->_retvalue = array_merge($this->yystack[$this->yyidx + -2]->minor,array($this->yystack[$this->yyidx + 0]->minor));
    }
#line 2702 "smarty_internal_templateparser.php"
#line 986 "smarty_internal_templateparser.y"
    function yy_r141(){
    $this->_retvalue = array_merge($this->yystack[$this->yyidx + -2]->minor,array(array_merge($this->yystack[$this->yyidx + -1]->minor,$this->yystack[$this->yyidx + 0]->minor)));
    }
#line 2707 "smarty_internal_templateparser.php"
#line 990 "smarty_internal_templateparser.y"
    function yy_r142(){
    $this->_retvalue = array(array_merge($this->yystack[$this->yyidx + -1]->minor,$this->yystack[$this->yyidx + 0]->minor));
    }
#line 2712 "smarty_internal_templateparser.php"
#line 994 "smarty_internal_templateparser.y"
    function yy_r143(){
    $this->compiler->assert_is_not_strict('@ is not allowed in templates', $this);
    $this->_retvalue = array($this->yystack[$this->yyidx + 0]->minor);
    }
#line 2718 "smarty_internal_templateparser.php"
#line 999 "smarty_internal_templateparser.y"
    function yy_r144(){
    $this->_retvalue =  array($this->yystack[$this->yyidx + 0]->minor);
    }
#line 2723 "smarty_internal_templateparser.php"
#line 1007 "smarty_internal_templateparser.y"
    function yy_r145(){
    $this->_retvalue = array_merge($this->yystack[$this->yyidx + -1]->minor,$this->yystack[$this->yyidx + 0]->minor);
    }
#line 2728 "smarty_internal_templateparser.php"
#line 1027 "smarty_internal_templateparser.y"
    function yy_r149(){
    $this->_retvalue = '==';
    }
#line 2733 "smarty_internal_templateparser.php"
#line 1031 "smarty_internal_templateparser.y"
    function yy_r150(){
    $this->_retvalue = '!=';
    }
#line 2738 "smarty_internal_templateparser.php"
#line 1035 "smarty_internal_templateparser.y"
    function yy_r151(){
    $this->_retvalue = '>';
    }
#line 2743 "smarty_internal_templateparser.php"
#line 1039 "smarty_internal_templateparser.y"
    function yy_r152(){
    $this->_retvalue = '<';
    }
#line 2748 "smarty_internal_templateparser.php"
#line 1043 "smarty_internal_templateparser.y"
    function yy_r153(){
    $this->_retvalue = '>=';
    }
#line 2753 "smarty_internal_templateparser.php"
#line 1047 "smarty_internal_templateparser.y"
    function yy_r154(){
    $this->_retvalue = '<=';
    }
#line 2758 "smarty_internal_templateparser.php"
#line 1051 "smarty_internal_templateparser.y"
    function yy_r155(){
    $this->_retvalue = '===';
    }
#line 2763 "smarty_internal_templateparser.php"
#line 1055 "smarty_internal_templateparser.y"
    function yy_r156(){
    $this->_retvalue = '!==';
    }
#line 2768 "smarty_internal_templateparser.php"
#line 1059 "smarty_internal_templateparser.y"
    function yy_r157(){
    $this->_retvalue = '%';
    }
#line 2773 "smarty_internal_templateparser.php"
#line 1063 "smarty_internal_templateparser.y"
    function yy_r158(){
    $this->_retvalue = '&&';
    }
#line 2778 "smarty_internal_templateparser.php"
#line 1067 "smarty_internal_templateparser.y"
    function yy_r159(){
    $this->_retvalue = '||';
    }
#line 2783 "smarty_internal_templateparser.php"
#line 1071 "smarty_internal_templateparser.y"
    function yy_r160(){
    $this->compiler->assert_is_not_strict('XOR is not supported in strict mode', $this);
    $this->_retvalue = ' XOR ';
    }
#line 2789 "smarty_internal_templateparser.php"
#line 1079 "smarty_internal_templateparser.y"
    function yy_r161(){
    $this->_retvalue = 'array('.$this->yystack[$this->yyidx + -1]->minor.')';
    }
#line 2794 "smarty_internal_templateparser.php"
#line 1087 "smarty_internal_templateparser.y"
    function yy_r163(){
    $this->_retvalue = $this->yystack[$this->yyidx + -2]->minor.','.$this->yystack[$this->yyidx + 0]->minor;
    }
#line 2799 "smarty_internal_templateparser.php"
#line 1091 "smarty_internal_templateparser.y"
    function yy_r164(){
    return;
    }
#line 2804 "smarty_internal_templateparser.php"
#line 1095 "smarty_internal_templateparser.y"
    function yy_r165(){
    $this->_retvalue = $this->yystack[$this->yyidx + -2]->minor.'=>'.$this->yystack[$this->yyidx + 0]->minor;
    }
#line 2809 "smarty_internal_templateparser.php"
#line 1099 "smarty_internal_templateparser.y"
    function yy_r166(){
    $this->_retvalue = '\''.$this->yystack[$this->yyidx + -2]->minor.'\'=>'.$this->yystack[$this->yyidx + 0]->minor;
    }
#line 2814 "smarty_internal_templateparser.php"
#line 1111 "smarty_internal_templateparser.y"
    function yy_r168(){
    $this->_retvalue = "''";
    }
#line 2819 "smarty_internal_templateparser.php"
#line 1115 "smarty_internal_templateparser.y"
    function yy_r169(){
    $this->_retvalue = $this->yystack[$this->yyidx + -1]->minor->to_smarty_php();
    }
#line 2824 "smarty_internal_templateparser.php"
#line 1120 "smarty_internal_templateparser.y"
    function yy_r170(){
    $this->yystack[$this->yyidx + -1]->minor->append_subtree($this->yystack[$this->yyidx + 0]->minor);
    $this->_retvalue = $this->yystack[$this->yyidx + -1]->minor;
    }
#line 2830 "smarty_internal_templateparser.php"
#line 1125 "smarty_internal_templateparser.y"
    function yy_r171(){
    $this->_retvalue = new _smarty_doublequoted($this, $this->yystack[$this->yyidx + 0]->minor);
    }
#line 2835 "smarty_internal_templateparser.php"
#line 1129 "smarty_internal_templateparser.y"
    function yy_r172(){
    $this->_retvalue = new _smarty_code($this, '(string)' . $this->compileVariable("'" . substr($this->yystack[$this->yyidx + 0]->minor,1) . "'"));
    }
#line 2840 "smarty_internal_templateparser.php"
#line 1133 "smarty_internal_templateparser.y"
    function yy_r173(){
    $this->_retvalue = new _smarty_code($this, '(string)'.$this->yystack[$this->yyidx + -1]->minor);
    }
#line 2845 "smarty_internal_templateparser.php"
#line 1137 "smarty_internal_templateparser.y"
    function yy_r174(){
    $this->_retvalue = new _smarty_code($this, '(string)('.$this->yystack[$this->yyidx + -1]->minor.')');
    }
#line 2850 "smarty_internal_templateparser.php"
#line 1141 "smarty_internal_templateparser.y"
    function yy_r175(){
    $this->_retvalue = new _smarty_tag($this, $this->yystack[$this->yyidx + -1]->minor);
    }
#line 2855 "smarty_internal_templateparser.php"
#line 1145 "smarty_internal_templateparser.y"
    function yy_r176(){
    $this->_retvalue = new _smarty_dq_content($this, $this->yystack[$this->yyidx + 0]->minor);
    }
#line 2860 "smarty_internal_templateparser.php"

    private $_retvalue;

    public function yy_reduce($yyruleno)
    {
        $yymsp = $this->yystack[$this->yyidx];
        if ($this->yyTraceFILE && $yyruleno >= 0
              && $yyruleno < count(self::$yyRuleName)) {
            fprintf($this->yyTraceFILE, "%sReduce (%d) [%s].\n",
                $this->yyTracePrompt, $yyruleno,
                self::$yyRuleName[$yyruleno]);
        }

        $this->_retvalue = $yy_lefthand_side = null;
        if (array_key_exists($yyruleno, self::$yyReduceMap)) {
            // call the action
            $this->_retvalue = null;
            $this->{'yy_r' . self::$yyReduceMap[$yyruleno]}();
            $yy_lefthand_side = $this->_retvalue;
        }
        $yygoto = self::$yyRuleInfo[$yyruleno]['lhs'];
        $yysize = self::$yyRuleInfo[$yyruleno]['rhs'];
        $this->yyidx -= $yysize;
        for ($i = $yysize; $i; $i--) {
            // pop all of the right-hand side parameters
            array_pop($this->yystack);
        }
        $yyact = $this->yy_find_reduce_action($this->yystack[$this->yyidx]->stateno, $yygoto);
        if ($yyact < self::YYNSTATE) {
            if (!$this->yyTraceFILE && $yysize) {
                $this->yyidx++;
                $x = new TP_yyStackEntry;
                $x->stateno = $yyact;
                $x->major = $yygoto;
                $x->minor = $yy_lefthand_side;
                $this->yystack[$this->yyidx] = $x;
            } else {
                $this->yy_shift($yyact, $yygoto, $yy_lefthand_side);
            }
        } elseif ($yyact == self::YYNSTATE + self::YYNRULE + 1) {
            $this->yy_accept();
        }
    }

    public function yy_parse_failed()
    {
        if ($this->yyTraceFILE) {
            fprintf($this->yyTraceFILE, "%sFail!\n", $this->yyTracePrompt);
        } while ($this->yyidx >= 0) {
            $this->yy_pop_parser_stack();
        }
    }

    public function yy_syntax_error($yymajor, $TOKEN)
    {
#line 119 "smarty_internal_templateparser.y"

    $this->internalError = true;
    $this->yymajor = $yymajor;
    $this->compiler->trigger_template_error();
#line 2922 "smarty_internal_templateparser.php"
    }

    public function yy_accept()
    {
        if ($this->yyTraceFILE) {
            fprintf($this->yyTraceFILE, "%sAccept!\n", $this->yyTracePrompt);
        } while ($this->yyidx >= 0) {
            $stack = $this->yy_pop_parser_stack();
        }
#line 112 "smarty_internal_templateparser.y"

    $this->successful = !$this->internalError;
    $this->internalError = false;
    $this->retvalue = $this->_retvalue;
#line 2938 "smarty_internal_templateparser.php"
    }

    public function doParse($yymajor, $yytokenvalue)
    {
        $yyerrorhit = 0;   /* True if yymajor has invoked an error */

        if ($this->yyidx === null || $this->yyidx < 0) {
            $this->yyidx = 0;
            $this->yyerrcnt = -1;
            $x = new TP_yyStackEntry;
            $x->stateno = 0;
            $x->major = 0;
            $this->yystack = array();
            array_push($this->yystack, $x);
        }
        $yyendofinput = ($yymajor==0);

        if ($this->yyTraceFILE) {
            fprintf($this->yyTraceFILE, "%sInput %s\n",
                $this->yyTracePrompt, $this->yyTokenName[$yymajor]);
        }

        do {
            $yyact = $this->yy_find_shift_action($yymajor);
            if ($yymajor < self::YYERRORSYMBOL &&
                  !$this->yy_is_expected_token($yymajor)) {
                // force a syntax error
                $yyact = self::YY_ERROR_ACTION;
            }
            if ($yyact < self::YYNSTATE) {
                $this->yy_shift($yyact, $yymajor, $yytokenvalue);
                $this->yyerrcnt--;
                if ($yyendofinput && $this->yyidx >= 0) {
                    $yymajor = 0;
                } else {
                    $yymajor = self::YYNOCODE;
                }
            } elseif ($yyact < self::YYNSTATE + self::YYNRULE) {
                $this->yy_reduce($yyact - self::YYNSTATE);
            } elseif ($yyact == self::YY_ERROR_ACTION) {
                if ($this->yyTraceFILE) {
                    fprintf($this->yyTraceFILE, "%sSyntax Error!\n",
                        $this->yyTracePrompt);
                }
                if (self::YYERRORSYMBOL) {
                    if ($this->yyerrcnt < 0) {
                        $this->yy_syntax_error($yymajor, $yytokenvalue);
                    }
                    $yymx = $this->yystack[$this->yyidx]->major;
                    if ($yymx == self::YYERRORSYMBOL || $yyerrorhit) {
                        if ($this->yyTraceFILE) {
                            fprintf($this->yyTraceFILE, "%sDiscard input token %s\n",
                                $this->yyTracePrompt, $this->yyTokenName[$yymajor]);
                        }
                        $this->yy_destructor($yymajor, $yytokenvalue);
                        $yymajor = self::YYNOCODE;
                    } else {
                        while ($this->yyidx >= 0 &&
                                 $yymx != self::YYERRORSYMBOL &&
        ($yyact = $this->yy_find_shift_action(self::YYERRORSYMBOL)) >= self::YYNSTATE
                              ){
                            $this->yy_pop_parser_stack();
                        }
                        if ($this->yyidx < 0 || $yymajor==0) {
                            $this->yy_destructor($yymajor, $yytokenvalue);
                            $this->yy_parse_failed();
                            $yymajor = self::YYNOCODE;
                        } elseif ($yymx != self::YYERRORSYMBOL) {
                            $u2 = 0;
                            $this->yy_shift($yyact, self::YYERRORSYMBOL, $u2);
                        }
                    }
                    $this->yyerrcnt = 3;
                    $yyerrorhit = 1;
                } else {
                    if ($this->yyerrcnt <= 0) {
                        $this->yy_syntax_error($yymajor, $yytokenvalue);
                    }
                    $this->yyerrcnt = 3;
                    $this->yy_destructor($yymajor, $yytokenvalue);
                    if ($yyendofinput) {
                        $this->yy_parse_failed();
                    }
                    $yymajor = self::YYNOCODE;
                }
            } else {
                $this->yy_accept();
                $yymajor = self::YYNOCODE;
            }
        } while ($yymajor != self::YYNOCODE && $this->yyidx >= 0);
    }
}
